<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xml="http://www.w3.org/XML/1998/namespace" xmlns:as="http://www.oasis-open.org/committees/cam"  xmlns:camed="http://jcam.org.uk/editor"
	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
	xmlns:look="http://jcam.org.uk/LookupLists">
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: David RR Webber, September, 2009 -->
	<!--
		Purpose  : Used by top down designer in CAM editor. Parses input CAM template file and expands dictionary structure references into new expanded CAM template.
		Inputs: Collection of lookup dictionary definitions.
		Original : 09/23/2009
		Initials : drrw
		Description : Parses input CAM template file and expands dictionary structure references into new expanded CAM template.  Makes inline expansion notes into comments.
		                     Uses default CAM template format, including any extensions.
		
		Updated  - Inits - Change
		09/23/09   drrw   Initial release 
		11/03/09   drrw   Update for LEXS dictionary processing
		11/03/09   drrw   Mask of enumeration list
		11/05/09   drrw   Enhanced child and parent item traversal
		11/06/09   drrw   Consolidate namespaces in output
		11/06/09   drrw   Write rules and annotations of child items
		11/07/09   drrw   Child item traversal
		11/09/09   drrw   Add enhanced structure traverse logic to rule and annotation lookup
		11/16/09   drrw   Handle dictionary multi-line context (&#xA;)
		11/17/09   drrw   Handle attribute definitions in dictionary
		11/17/09   drrw   Collect structure items from all context definitions
		11/25/09   drrw   Enhance rule and annotation structure logic		
		11/25/09   drrw   Add attributes to structure
		11/25/09   drrw   Add attributes to rules
		11/27/09   drrw   Duplicates rules logic, make full subpath
		11/27/09   drrw   Carry comments forward, not just the element tag itself
		11/27/09   drrw   Handle existing comments from as:Extension or as:include
		11/27/09   drrw   Fix expanding by annotation Type label
		11/28/09   drrw   Fix ignore of expanded label
		11/28/09   drrw   Output expanded annotation for every node
		11/29/09   drrw   as:Extension within as:Include handling and expansion messages
		12/02/09   drrw   Add path parameter for use with as:include expanding
		12/05/09   drrw   Handle duplicate and variations in context paths for same structure items
		12/14/09   drrw   Type expansion - handle parent node with namespace
		12/16/09   drrw   Tweak to adjust context to annotation Type expansion in children matching
		12/23/09   drrw    Leaf parent context handling fix
		12/30/09   drrw    SetTypeSample value outputting of datetime
		12/30/09   drrw    Correct parent path logic in child parent leaf node processing
		12/30/09   drrw    Singleton node type sample outputting when leaf node
		12/31/09   drrw    Ignore ABIE nodes with no child items
		12/31/09   drrw    Provide ability to filter only on nodes that exist within parent
		12/31/09   drrw    Enhance performance by filter on matching context nodes within parent
		01/01/10   drrw    Example value output for BBIE item
		01/01/10   drrw    Rules outputting for partial structure items
		01/04/10   drrw    Fix for multiple child items with same name different context path
		01/07/10   drrw    Refine handling in prevent duplicates occurring logic
		02/04/10   drrw    Add additional facet type matching
		02/05/10   drrw    Add children handling logic 
		02/18/10   drrw    Path fix for @ attribute names XPath
		02/18/10   drrw    Enhance types handling logic to permit extension
		02/18/10   drrw    Enhance annotation expanded from path lookups
		02/25/10   drrw    Fix multiple item retrieval
		02/25/10   drrw    Refined attributes matching on short context paths
		02/26/10   drrw    Unmatched namespace prefix in dictionary control file
		02/26/10   drrw    Fix parent path to not include as:Structure - use rootElement
		02/26/10   drrw    Process simple root structure with no namespace
		02/27/10   drrw    Correct attributes sequence check on output
		03/02/10   drrw    Update attribute ASBIE matching in rules / anno import
		03/02/10   drrw    Correct XPath from //@ to /@ in setFacetRules
		03/02/10   drrw    Correct XPath from @ to /@ in annotations
		03/04/10   drrw    Fix to sequence check for child:: parent reference
		03/13/10   drrw    Look up of dictionary definition details on ABIE path item
		03/14/10   drrw    Apply same changes to annotations as rules look up
		03/16/10   drrw    setLength for strings with mask length 
		03/18/10   drrw    Make context loop on type references unique items only
		03/18/10   drrw    Sequence check on BBIE items ignore children
		03/23/10   drrw    Control restriction rules outputting
		04/04/10   drrw    Enhance duplicate rule checking
		04/25/10   drrw    Resolve recursion issue in import mode items
		05/06/10   drrw    Fix crash with multiple dictionary items selected
		05/06/10   drrw    Fix annotations XPath issue with child attributes
		05/12/10   drrw    Tweak recursion check name matching
		05/12/10   drrw    Fix annotation attribute paths longer than parent/attribute
		05/12/10   drrw    Fix as:Structure in annotation path
		05/17/10   drrw    Correct recursive skipping elements when not nested items
		08/16/10   drrw    Duplicate rule performance enhancement - not implemented - original code kept
		09/12/10   drrw    Bug fix for mask string handling
		01/16/11   drrw    Verify restrictValues list quotes inserting works correctly
		01/16/11   drrw    Dictionary type of token or language should be datatype rule 
		01/16/11   drrw    Strict and Lax mode handling
		02/19/11   drrw    Fix for count() of empty string error
	-->
	<xsl:param name="outputFilePath"/>
	<xsl:param name="componentsRespositoryFilepath"/>
	<xsl:param name="inputFilePath"/>
	<xsl:param name="restrictionRules">Dates, Codes, Numbers</xsl:param><!-- Dates, Codes, Numbers, Strings, All, None -->
	<xsl:param name="runMode"></xsl:param><!-- Strict, Lax -->
	<xsl:output method="xml" indent="yes"/>
	<xsl:variable name="xslt-version">1.30</xsl:variable>
	<xsl:variable name="nlchar" select="'&#xA;'"/>
	<xsl:variable name="debug">0</xsl:variable><!-- 0, 1, 2, 3, 4 or 5 -->
	<xsl:variable name="rootElement" select="as:CAM/as:AssemblyStructure/as:Structure/*[1]/name()"/>
	
	<xsl:variable name="useStrings" select="if(contains($restrictionRules,'Strings') or(contains($restrictionRules,'All'))) then 'true' else 'false'"/>
	<xsl:variable name="useDates" select="if(contains($restrictionRules,'Dates') or(contains($restrictionRules,'All'))) then 'true' else 'false'"/>
	<xsl:variable name="useCodes" select="if(contains($restrictionRules,'Codes') or(contains($restrictionRules,'All'))) then 'true' else 'false'"/>
	<xsl:variable name="useNumbers" select="if(contains($restrictionRules,'Numbers') or(contains($restrictionRules,'All'))) then 'true' else 'false'"/>
	<xsl:variable name="useAll" select="if(contains($restrictionRules,'All')) then 'true' else 'false'"/>
	
	<xsl:variable name="componentRepository" select="document($componentsRespositoryFilepath)"/>
	
	<xsl:variable name="dictionary">
		<xsl:for-each select="$componentRepository//camed:dictionary">
			<xsl:variable name="thisFile" select="@location"/>
			<components shortname="{@shortname}" namespaces="{@namespaces}">	
				<!-- xsl:copy-of select="document($thisFile)"/ -->
				<xsl:variable name="tempDictionary" select="document($thisFile)"/>
				<ITEMSlookup>
					<xsl:comment><xsl:value-of select="$tempDictionary/ITEMSlookup/comment()"></xsl:value-of></xsl:comment>
				<!-- now unpack the context -->
				<xsl:for-each select="$tempDictionary//Property">
					<xsl:variable name="propertyDetails" select="."/>
					<xsl:variable name="uniqueContext" select="tokenize(./@context,'&#xA;')"/>
					<xsl:for-each select="$uniqueContext">
						<xsl:if test="string-length(.) &gt; 0">
							<xsl:element name="Property">
								<xsl:for-each select="$propertyDetails/@*">
									<xsl:if test="not(name()='context')">
										<xsl:attribute name="{name()}" select="."/>
									</xsl:if>
								</xsl:for-each>
								<xsl:attribute name="context" select="."/>
							</xsl:element>
						</xsl:if>
					</xsl:for-each>	
				</xsl:for-each>
				</ITEMSlookup>
			</components>
		</xsl:for-each>
	</xsl:variable>
	
	<xsl:variable name="namespaces" select="$dictionary/components/@namespaces"/>
	
	<xsl:variable name="namespacesList">
		<xsl:for-each select="$namespaces"><xsl:value-of select="."/>|</xsl:for-each>
	</xsl:variable>
	
	<xsl:variable name="namespaceLookup">
		<namespaces>
		<xsl:for-each select=".//namespace::*">
			<xsl:if test="not(name()='w2e')">
				<namespace name="{name()}" select="{.}"/>
			</xsl:if>
		</xsl:for-each>
		</namespaces>
	</xsl:variable>
	
	<xsl:variable name="itemAnnotations">
		<!-- xsl:message>anno inputFilePath: <xsl:value-of select="$inputFilePath"/></xsl:message -->
		<xsl:choose>
			<xsl:when test="//as:Extension"><xsl:copy-of select="//as:Extension"/></xsl:when>
			<xsl:when test="//as:Include/as:Extension"><xsl:copy-of select="//as:Include/as:Extension"/></xsl:when>
			<xsl:otherwise>
				<xsl:variable name="annoIncludes"><xsl:copy-of select="//as:include"/></xsl:variable>
				<xsl:for-each select="$annoIncludes/as:include">
					<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/><xsl:value-of select="."/></xsl:variable>
					<xsl:if test="contains(.,'.anno')"><xsl:copy-of select="document(replace(replace($filePath,'C:\\','\\'),'\\','/'))"/></xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="expandedStructure">
		<xsl:apply-templates select="//as:AssemblyStructure/as:Structure" mode="structure"/>
	</xsl:variable>
	
	<xsl:variable name="newRules">
		<xsl:apply-templates select="$expandedStructure/as:Structure" mode="structureRules"/>
	</xsl:variable>
	
	<xsl:variable name="newAnno">
		<xsl:apply-templates select="$expandedStructure/as:Structure" mode="structureAnno"/>
	</xsl:variable>
	
	<xsl:variable name="expansionItems">
		<expander>
		   <xsl:for-each select="$itemAnnotations//as:Extension/camed:annotation">
		   	<xsl:choose>
		   		<xsl:when test="camed:documentation[@type='Expanded']">
		   			<xsl:element name="camed:annotation">
		   				<xsl:attribute name="item" select="replace(@item,'//','/')"></xsl:attribute>
		   			</xsl:element>
		   		</xsl:when>
		   		<xsl:when test="camed:documentation[@type='Type']">
		   			<xsl:element name="camed:expand">
		   				<xsl:attribute name="item" select="@item"></xsl:attribute>
		   				<xsl:attribute name="from" select="camed:documentation[@type='Type']"></xsl:attribute>
		   			</xsl:element>
		   		</xsl:when>
		   		<xsl:otherwise/>
		   	</xsl:choose>
		   </xsl:for-each>
		</expander>
	</xsl:variable>
	
	<xsl:strip-space elements="*"/>
	
	<xsl:template match="/">
		<xsl:variable name="templateNS" select=".//namespace::*"/>
		<xsl:message>CAM Expander starting <xsl:value-of select="dateTime(current-date(),current-time())"/></xsl:message>
		<xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
		<xsl:message>Using input control file: <xsl:value-of select="$componentsRespositoryFilepath"/> file.</xsl:message>
		<xsl:message>Dictionaries: <xsl:value-of select="$componentRepository//camed:dictionary/@location"/></xsl:message>
		<xsl:message>Run Mode: <xsl:value-of select="$runMode"/></xsl:message>
		<xsl:message>From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/></xsl:message>
		<xsl:if test="count($componentRepository//camed:dictionary/@location) = 0"><xsl:message terminate="yes">Error: input repository not found!</xsl:message></xsl:if>
		
		<!-- xsl:message>dictionary: <xsl:copy-of select="$dictionary//Property[@CCTS='ASBIE']"/></xsl:message -->
		
		<as:CAM xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:xml="http://www.w3.org/XML/1998/namespace" CAMlevel="{as:CAM/@CAMlevel}" version="{as:CAM/@version}">
			<xsl:for-each select=".//namespace::*">
				<xsl:if test="not(name()='w2e')">
					<xsl:namespace name="{name()}" select="."/>
				</xsl:if>
			</xsl:for-each>

<xsl:comment xml:space="preserve">
	CAM Expander 
	Version: <xsl:value-of select="$xslt-version"/>
	Date: <xsl:value-of select="dateTime(current-date(),current-time())"/>
	Using input control file: <xsl:value-of select="$componentsRespositoryFilepath"/> file.
	Dictionaries: <xsl:value-of select="$componentRepository//camed:dictionary/@location"></xsl:value-of>
	Apply restrictions: <xsl:value-of select="$restrictionRules"/>
	From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/>

</xsl:comment>

			<xsl:message>Writing structure</xsl:message>
			<xsl:message> (path trace diagnostics)</xsl:message>
			<xsl:apply-templates select="//as:Header"/>
			<xsl:element name="as:AssemblyStructure"><!-- output the expanded Structure details -->
				<xsl:for-each select="//as:AssemblyStructure/@*">
					<xsl:attribute name="{name()}" select="."/>
				</xsl:for-each>
				<xsl:copy-of select="$expandedStructure"/>
			</xsl:element>
			<xsl:message>structure completed</xsl:message>
			<xsl:message>Writing rules</xsl:message>
			<xsl:apply-templates select="//as:BusinessUseContext"/>
			<xsl:message>completed rules</xsl:message>
			<!-- xsl:message>annotations: [<xsl:copy-of select="$itemAnnotations"/>]</xsl:message -->
			<xsl:if test="$expansionItems/expander/camed:expand">
				<xsl:message>expansion details:</xsl:message>
				<xsl:for-each select="$expansionItems/expander/camed:expand">
					<xsl:message> expand: <xsl:value-of select="@item"/> - with type - <xsl:value-of select="@from"/></xsl:message>
				</xsl:for-each>
			</xsl:if>
			<!-- xsl:message>includes: [<xsl:copy-of select="//as:include"/>]</xsl:message -->
			<xsl:message>Writing annotations</xsl:message>
			<xsl:choose>
				<xsl:when test="//as:Extension"><xsl:apply-templates select="//as:Extension"/></xsl:when>
				<xsl:otherwise>
					<as:Extension name="uk.org.jcam.camed.extensions.StructureAnnotations">
						<xsl:copy-of select="$newAnno/as:MoreAnno/*"/>
					</as:Extension>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:message>completed annotations</xsl:message>
			<!-- xsl:apply-templates select="//as:include"/ --><!-- disable for now, CAMeditor will recreate includes on save -->
			<xsl:choose>				
				<xsl:when test="count($componentRepository/as:Extension) &gt; 0"><!-- debugging only - xsl:copy-of select="$componentRepository"/ --></xsl:when>
				<xsl:otherwise><xsl:comment>WARNING: No dictionary repository found for file: [<xsl:value-of select="$componentsRespositoryFilepath"/>]</xsl:comment>
				</xsl:otherwise>
			</xsl:choose>
			<!-- debugging only - xsl:copy-of select="$dictionary"/ -->
			<!-- debugging only - xsl:copy-of select="$expandedItems"/ -->
		</as:CAM>
		<xsl:message>CAM Expander finished processing.</xsl:message>
	</xsl:template>

	<xsl:template match="as:Header">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:AssemblyStructure">
		<xsl:copy>
			<xsl:apply-templates select="as:Structure"/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="as:BusinessUseContext">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="as:Extension">
		<as:Extension name="{@name}">
			<xsl:apply-templates/>
			<xsl:copy-of select="$newAnno/as:MoreAnno/*"/>
		</as:Extension>
	</xsl:template>
	
	<xsl:template match="camed:annotation">
		<camed:annotation item="{@item}">
			<xsl:apply-templates/>
		</camed:annotation>
	</xsl:template>
	
	<xsl:template match="camed:documentation">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="look:LookupList">
		<xsl:copy>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="look:item">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:Structure">
		<as:Structure ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates mode="structure"/>
		</as:Structure>
	</xsl:template>
	
	<xsl:template match="as:Structure" mode="structureRules">
		<as:MoreRules ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureRules"/>
		</as:MoreRules>
	</xsl:template>

	<xsl:template match="as:Structure" mode="structureAnno">
		<as:MoreAnno ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureAnno"/>
		</as:MoreAnno>
	</xsl:template>

	<xsl:template match="as:Rules">
		<as:Rules>
			<as:default><xsl:apply-templates select="as:default"/></as:default>
			<xsl:if test="as:context">
			<as:context>
				<xsl:attribute name="condition"><xsl:value-of select="as:context/@condition"/></xsl:attribute>
				<xsl:apply-templates select="as:context"/>
			</as:context>
			</xsl:if>
		</as:Rules>
	</xsl:template>
	
	<xsl:template match="as:default">
		<as:context>
			<xsl:apply-templates select="as:context"/>
			<!-- Now insert new rules -->
			<xsl:variable name="uniqueRules" select="distinct-values($newRules/as:MoreRules/as:constraint/@action)"/>
			<!-- xsl:variable name="uniqueRules">
				<xsl:for-each select="$newRules/as:MoreRules/as:constraint">
				<xsl:variable name="this" select="if(@action) then @action else '%NONE%'"/>
				<xsl:variable name="count" select="count($newRules/as:MoreRules/as:constraint[@action=$this][position() &lt; 3])"/>
				<xsl:choose> 
					<xsl:when test="$count &gt; 1">
						<xsl:variable name="chk" select="./following::*[@action=$this][1]"/ --><!-- optimization of duplicate removal -->
						<!-- xsl:message> <xsl:value-of select="$this"/> - <xsl:value-of select="position()"/> | <xsl:value-of select="count($chk[@action=$this])"/></xsl:message -->
						<!-- xsl:choose>
							<xsl:when test="count($chk[@action=$this][1]) &gt; 0" --><!-- xsl:comment>duplicate: <xsl:value-of select="$this"/></xsl:comment --><!-- /xsl:when>
							<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
						</xsl:choose>					
					</xsl:when>
					<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
				</xsl:choose>	
			</xsl:for-each>
			</xsl:variable -->
			<!-- Check for short / long path duplicates -->
			<xsl:variable name="dedupUniqueRules">
				<xsl:for-each select="$uniqueRules"><!-- use $uniqueRules//@action for performance duplicate removal -->
					<xsl:variable name="checkRules" select="."/>
					<xsl:variable name="delimiter" select="if(contains(.,',')) then ',' else ')'"/>
					<xsl:variable name="ruleXpath" select="substring-before(substring-after(.,'(/'),$delimiter)"/>
					<xsl:variable name="ruleXpathComma" select="concat($ruleXpath,',')"/>
					<xsl:variable name="ruleType" select="substring-before(.,'(')"/>
					<xsl:variable name="pathTokens" select="tokenize($ruleXpath,'/')"/>
					
					<xsl:variable name="shortPath">
					      <xsl:choose>
					      	<xsl:when test="count($pathTokens) &lt; 4"><xsl:value-of select="$ruleXpath"/></xsl:when>
					      	<xsl:when test="contains($ruleXpath,'@') and count($pathTokens) &lt; 5"><xsl:value-of select="$ruleXpath"/></xsl:when>
					      	<xsl:otherwise/>
					      </xsl:choose>	
					</xsl:variable>
					<xsl:choose>
						<xsl:when test="$shortPath = ''">
							<xsl:element name="as:constraint">
								<xsl:attribute name="action" select="."/>
							</xsl:element>
						</xsl:when>
						<xsl:otherwise>
							<!-- xsl:message> <xsl:value-of select="$ruleXpath"/> - <xsl:value-of select="$delimiter"/> - <xsl:value-of select="$ruleType"/></xsl:message -->
							<xsl:variable name="findRules" select="$newRules/as:MoreRules/as:constraint[contains(@action,concat($ruleXpath,$delimiter))][starts-with(@action,$ruleType)][not(starts-with(@action,concat($ruleType,'(/',$ruleXpath)))]"/>
							<xsl:choose>
								<xsl:when test="count($findRules/@action) &gt; 0"><xsl:if test="count($findRules/@action) &gt; 1"><xsl:message> duplicate rule <xsl:value-of select="$ruleType"/> - <xsl:value-of select="$shortPath"/> - <xsl:copy-of select="count($findRules/@action)"/></xsl:message></xsl:if></xsl:when>
								<xsl:otherwise>
									<xsl:element name="as:constraint">
										<xsl:attribute name="action" select="."/>
									</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:variable>
			<xsl:copy-of select="$dedupUniqueRules"/>
			<xsl:message> total rules added: <xsl:value-of select="count($dedupUniqueRules/*)"/></xsl:message>
		</as:context>
		
	</xsl:template>
	
	<xsl:template match="as:context">
		<xsl:apply-templates/>
	</xsl:template>
	
	<xsl:template match="as:constraint">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy>
			<xsl:if test="string-length($include) &gt; 0">
				<xsl:attribute name="as:include" select="$include"/>
			</xsl:if>
			<xsl:apply-templates mode="start"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="@*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:include" mode="start">
		<!-- If there are any includes expand them to inline structure components (TBD the CAMeditor may already do this in Java -->
		<!-- TBD This part may need attention -->
		<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/>/<xsl:value-of select="."/></xsl:variable>
		<xsl:variable name="include" select="document($filePath)"/>
		<xsl:apply-templates select="$include/*" mode="start">
			<xsl:with-param name="include" select="."/>
		</xsl:apply-templates>		
		
	</xsl:template>
	
	<xsl:template match="as:include">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="*" mode="structure">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="parentItem" select="if(../name()='as:Structure') then '' else ../name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parentPath" select="replace(concat('/',$parentItem,$contextMatch,'/'),'//','/')"/>
		<!-- find virtual parent path, this is the parent in the dictionary that precedes this item to be expanded -->
		<xsl:if test="$debug &gt; 0"><xsl:message>- <xsl:value-of select="$thisItem"/>[<xsl:value-of select="$parentItem"/>][<xsl:value-of select="$rootElement"/>]</xsl:message></xsl:if>
		<xsl:variable name="virtualParentPath">
			<xsl:variable name="DictionaryPathMatches" select="$dictionary//components/ITEMSlookup/Property[contains(@context,$contextMatch)][not(contains(@context,'@'))]/@context"/>
			<xsl:variable name="uniquePaths" select="distinct-values($DictionaryPathMatches)"/>
			<xsl:variable name="parentCandidates">
				<xsl:for-each select="$uniquePaths">
					<xsl:variable name="chkPosition" select="tokenize(.,'/')"/>
					<!-- xsl:message>[<xsl:value-of select="$chkPosition[3]"/>][<xsl:value-of select="$thisItem"/>]-[<xsl:value-of select="$chkPosition[2]"/>]</xsl:message -->
					<xsl:if test="$chkPosition[3]=$thisItem"><this item="{$chkPosition[2]}"/></xsl:if>
				</xsl:for-each>
			</xsl:variable>
			<xsl:value-of select="concat('/',distinct-values($parentCandidates/this/@item)[1],'/',$thisItem)"/>
		</xsl:variable>
		<xsl:variable name="noNSname" select="if(contains($thisItem,':')) then substring-after($thisItem,':') else $thisItem"/>
		<xsl:variable name="noNSmatch" select="concat('/',$noNSname)"/>
		<!-- Choose to lookup item structure in dictionary, or simply use "as is"  -->
		<xsl:choose>
			<xsl:when test="name()='as:Structure'">
				<xsl:variable name="actualRootNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>as:Structure here: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				<xsl:element name="{name()}" namespace="{$actualRootNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:message>- structure of: <xsl:value-of select="./*/name()"/></xsl:message>
					<xsl:variable name="newStructure" select="./*"/>
					<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="./*/name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
					<xsl:element name="{./*/name()}" namespace="{$actualNS}"><!-- Insert root element, then process children -->
						<xsl:apply-templates select="$newStructure/attribute::*" mode="insert"/>
						<xsl:apply-templates mode="structure" select="$newStructure/./*"/>
					</xsl:element>
				</xsl:element>
			</xsl:when>
			<xsl:when test="contains($namespacesList,$thisNS) or($expansionItems/expander/camed:expand[ends-with(@item,$contextMatch)])"> 
				<!-- Lookup in dictionary -->
				
				<xsl:variable name="importStructure">
					<xsl:choose>
						<!-- ignore items already expanded -->
						<xsl:when test="$expansionItems/expander/camed:annotation[ends-with(@item,$contextMatch)]"><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:when>
						<xsl:when test="$expansionItems/expander/camed:expand[ends-with(@item,$contextMatch)]">
							<!-- expand each item with Type annotation -->
							
							<xsl:variable name="expandThis" select="$expansionItems/expander/camed:expand[ends-with(@item,$contextMatch)]/@from"/>
							<xsl:message> sub-path: <xsl:value-of select="$thisItem"/> of type <xsl:value-of select="$expandThis"/></xsl:message>
							<xsl:variable name="baseProperty" select="$dictionary//components/ITEMSlookup/Property[@item=$expandThis][not(contains(@context,'@'))]"/>
							<!-- xsl:message>[<xsl:copy-of select="$baseProperty"/>]</xsl:message -->
							
							<xsl:element name="Property">
								<xsl:attribute name="item" select="$thisItem"/>
								<xsl:for-each select="$baseProperty//@*">
									<xsl:choose>
										<xsl:when test="name()='item'"/>
										<xsl:when test="name()='context'"><xsl:attribute name="{name()}" select="concat('/',$thisItem)"/></xsl:when>
										<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
									</xsl:choose>
								</xsl:for-each>
							</xsl:element>
							<!-- xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[contains(@context,concat($expandThis,'/'))][not(contains(@context,'@'))]"/ -->
							<xsl:variable name="matchingProperties" select="$dictionary//components/ITEMSlookup/Property[contains(@context,concat($expandThis[1],'/'))][not(contains(@context,'@'))]"/>
							<xsl:variable name="uniqueMatchingItems" select="distinct-values($matchingProperties//@item)"/>
							<!-- xsl:message>Unique items - <xsl:value-of select="$uniqueMatchingItems"/></xsl:message -->
							<xsl:for-each select="$uniqueMatchingItems">
								<xsl:variable name="thisItemName" select="."/>
								<xsl:for-each select="$matchingProperties[@item=$thisItemName][1]">
								<xsl:message>   | <xsl:value-of select="@item"/></xsl:message>
								<xsl:element name="Property">
									<xsl:for-each select="@*">
										<xsl:choose>
											<xsl:when test="name()='context'">
												<!-- xsl:variable name="adjustContextPath"> 
													<xsl:variable name="tryThisPath" select="replace(.,concat('/',$expandThis,'/'),concat('/',$thisItem,'/'))"/>
													<xsl:choose>
														<xsl:when test="$dictionary//components/ITEMSlookup/Property[contains(@context,$tryThisPath)]"><xsl:value-of select="$tryThisPath"/></xsl:when>
														<xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
													</xsl:choose>
												</xsl:variable -->
												<xsl:attribute name="{name()}" select="replace(.,concat('/',$expandThis,'/'),concat('/',$thisItem,'/'))"/>
											</xsl:when>
											<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
										</xsl:choose>
									</xsl:for-each>
								</xsl:element>
								</xsl:for-each>
							</xsl:for-each>
						</xsl:when>
						<xsl:when test="$dictionary//components/ITEMSlookup/Property[@item=$thisItem]">
							
							<xsl:choose>
								<xsl:when test="count($dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]) &gt; 0">
									
									<xsl:if test="$debug &gt; 2"><xsl:message>  } <xsl:value-of select="$contextMatch"/> dictionary match (<xsl:value-of select="$virtualParentPath"/>)</xsl:message></xsl:if>		
									<!-- xsl:message>leaves: <xsl:value-of select="count($dictionary//components/ITEMSlookup/Property[starts-with(@context,$virtualParentPath)][not(contains(@context,'@'))][not(@item=$thisItem)])"/></xsl:message -->
									
									<!-- now output these items - but do modify of context to replace virtual context with actual context instead -->
									
									<xsl:variable name="theseChildrenMembers" select="child::node()"/>
									
									<xsl:variable name="childNames" select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem]/@children"/>
									
									<xsl:variable name="ChildElementNames">
										<xsl:choose>
											<xsl:when test="not($childNames[1]='')">
												<xsl:for-each select="tokenize(substring-after($childNames[1],$nlchar),$nlchar)">
													<xsl:if test="not(.='')">|<xsl:value-of select="."/>|</xsl:if>
												</xsl:for-each>
											</xsl:when>
											<xsl:otherwise>
												<xsl:for-each select="$theseChildrenMembers/.">|<xsl:value-of select="name()"/>|</xsl:for-each>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
									
									<!-- xsl:message>names: <xsl:value-of select="$virtualParentPath"/> - <xsl:value-of select="$ChildElementNames"/></xsl:message>
									<xsl:message>children: <xsl:value-of select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem]/@children"/>]</xsl:message -->
									
									<xsl:for-each select="$dictionary//components/ITEMSlookup/Property[starts-with(@context,concat($virtualParentPath,'/'))][not(contains(@context,'@'))][not(@item=$thisItem)]">
										<!-- xsl:message>   ..<xsl:value-of select="@item"/> - <xsl:value-of select="@context"/> -[<xsl:value-of select="$virtualParentPath"/>]</xsl:message -->
										<xsl:if test="$ChildElementNames='||' or(contains($ChildElementNames,concat('|',@item,'|')))">
										<!-- Adjust context to match items inside this parent -->
										<xsl:element name="Property">
											<xsl:for-each select="@*">
												<xsl:choose>
													<xsl:when test="name()='context'">
														<xsl:attribute name="context" select="concat($contextMatch,substring-after(.,$contextMatch))"/>
														<!-- xsl:message>new context <xsl:value-of select="concat($contextMatch,substring-after(.,$contextMatch))"/></xsl:message -->
													</xsl:when>
													<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
												</xsl:choose>	
											</xsl:for-each>
										</xsl:element>
										</xsl:if>
									</xsl:for-each>
									<!-- Now select context child items -->
									<xsl:variable name="childItemsList">
										<xsl:choose>
											<xsl:when test="string-length($ChildElementNames) &lt; 3"><xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[starts-with(@context,concat($contextMatch,'/'))][not(contains(@context,'@'))]"/></xsl:when>
											<xsl:otherwise>
												<!-- Now get nested children -->
												<xsl:for-each select="tokenize(replace($ChildElementNames,'\|\|','|'),'\|')">
													<xsl:variable name="thisNamePath" select="concat('/',.,'/')"/>
													<xsl:variable name="thisItemName" select="."/>
													<xsl:variable name="thisNameParentPath" select="concat($contextMatch,'/',.)"/>
													<xsl:if test="string-length($thisNamePath) &gt; 2">
														<!-- Find matching children and context items -->
														<xsl:variable name="thisContextOnes" select="$dictionary//components/ITEMSlookup/Property[starts-with(@children,$parentPath)][contains(@context,$thisNamePath) or(ends-with(concat(@context,'/'),$thisNamePath))][not(contains(@context,'@'))]"></xsl:variable>
														<xsl:variable name="thisContextMore" select="$dictionary//components/ITEMSlookup/Property[@CCTS='BBIE'][ends-with(@context,$thisNameParentPath)][not(contains(@context,'@'))]"></xsl:variable>
														<xsl:variable name="thisContextLax">
															<xsl:choose>
																<xsl:when test="$runMode='Lax'"><xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@item = $thisItemName][1][not(contains(@context,'@'))]"/></xsl:when>
																<xsl:otherwise/>
															</xsl:choose>
														</xsl:variable>
														
														<xsl:if test="not($thisContextOnes='') and(count($thisContextOnes) = 0 and(count($thisContextMore) = 0))"><xsl:message> WARNING: no context match of <xsl:value-of select="$thisNamePath"/> with <xsl:value-of select="$parentPath"/></xsl:message></xsl:if>
														<xsl:copy-of select="$thisContextOnes"/>
														<xsl:if test="not($thisContextOnes='') and(count($thisContextOnes = 0) and($runMode = 'Lax') and(not(contains($thisItemName,'@'))))"><!-- Fix of count() of empty string -->
															<xsl:variable name="thisContextLaxContext" select="$thisContextLax[1]/Property/@context"/>
															<xsl:variable name="thisLaxParentPath" select="substring-before($thisContextLaxContext[1],$thisItemName)"/>
															<xsl:message> LAX MODE: using <xsl:value-of select="$thisItemName"/> with <xsl:value-of select="$thisContextLaxContext[1]"/></xsl:message>
															<xsl:copy-of select="$thisContextLax[1]"/>
															
															<xsl:variable name="thisContextMoreLax" select="$dictionary//components/ITEMSlookup/Property[@CCTS='BBIE'][ends-with(@context,$thisLaxParentPath)][not(contains(@context,'@'))]"/>
															<xsl:copy-of select="$thisContextMoreLax"/>
														</xsl:if>
														<xsl:if test="count($thisContextOnes) = 0">
															<xsl:copy-of select="$thisContextMore"/>
														</xsl:if>
																<!-- xsl:if test="$thisNamePath = '/lexs:SRMessageMetadata/'"><xsl:message> SRMessageMetadata <xsl:copy-of select="$thisContextOnes"/></xsl:message></xsl:if -->
																<!-- xsl:message><xsl:for-each select="$thisContextOnes"> - <xsl:value-of select="./@item"/></xsl:for-each></xsl:message -->
																<!-- xsl:message> / <xsl:value-of select="."/> [<xsl:value-of select="$parentPath"/>] - <xsl:value-of select="$thisNamePath"/></xsl:message -->
													</xsl:if>
												</xsl:for-each>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
									<!-- Dedup that list -->
									<xsl:variable name="uniqueChildItems" select="distinct-values($childItemsList/Property/@item)"/>
									<xsl:for-each select="$uniqueChildItems">
										<xsl:variable name="thisChildItem" select="."/>
										<xsl:copy-of select="$childItemsList/Property[@item=$thisChildItem]"/><!-- note: this gets all child context occurences -->
												<!-- xsl:message>   ~ <xsl:value-of select="$childItemsList/Property[@item=$thisChildItem][1]/@item"/> properties: <xsl:value-of select="count($childItemsList/Property[@item=$thisChildItem])"/></xsl:message -->
												<!-- xsl:if test="$thisChildItem = 'lexs:DomainAttribute'"><xsl:message>[<xsl:copy-of select="$childItemsList/Property[@item=$thisChildItem]"/>]</xsl:message></xsl:if -->
									</xsl:for-each>
								</xsl:when>
								<xsl:otherwise><!-- Its a child element within a parent -->
									<xsl:variable name="childPathMatch" select="concat($contextMatch,'/')"/>
									<xsl:variable name="tempDefinition">
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem][1]"/>
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[contains(@context,$childPathMatch)][not(contains(@context,'@'))]"/>
									</xsl:variable>
									<xsl:element name="Property">
										<xsl:for-each select="$tempDefinition//@*">
											<xsl:choose>
												<xsl:when test="name()='context'">
													<xsl:attribute name="context" select="concat('/',$thisItem,substring-after(.,$thisItem))"/>
												</xsl:when>
												<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
											</xsl:choose>	
										</xsl:for-each>
									</xsl:element>
									<!-- xsl:message>  - regular match</xsl:message -->
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				
				<!-- xsl:if test="$thisItem='lexs:Capabilities'" --><!-- xsl:message>Structure: [<xsl:value-of select="$thisItem"/>][<xsl:copy-of select="$importStructure"/>]</xsl:message --><!-- /xsl:if -->
				
				<xsl:choose>
					<xsl:when test="$importStructure/no_match"><!-- Output structure item as is -->
						<xsl:if test="$debug &gt; 0"><xsl:message> no matching: <xsl:value-of select="@item"/></xsl:message></xsl:if>
						<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="$namespace-uri"/></xsl:call-template></xsl:variable>
						<xsl:element name="{name()}" namespace="{$actualNS}">
							<xsl:apply-templates select="attribute::*" mode="insert"/>
							<xsl:if test="not(exists(*)) and string-length(text()) &gt; 0"><xsl:value-of select="text()"/></xsl:if><!-- insert default value text -->
							<xsl:if test="as:include">
								<xsl:apply-templates mode="include"/>
							</xsl:if>
							<xsl:message>- <xsl:value-of select="name()"/></xsl:message>
							<xsl:apply-templates mode="structure"/>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise><!-- Output structure details from dictionary definition -->
						<xsl:variable name="uniqueContext" select="distinct-values($importStructure/Property/@context)"/>
						<xsl:variable name="dedupStructure">
							<xsl:for-each select="$uniqueContext">
								<xsl:variable name="thisContext" select="."/>
								<xsl:copy-of select="$importStructure/Property[@context=$thisContext][1]"/><!-- Match first context -->
							</xsl:for-each>
						</xsl:variable>
						
						<!-- Look to see if there are missing child context items of leaf nodes, and add them if so -->
						
						<xsl:variable name="leafPaths" select="distinct-values($dedupStructure/Property/@context)"/>
						
						<!-- xsl:message> show paths [<xsl:copy-of select="$leafPaths"/>]</xsl:message -->
						
						<xsl:variable name="leafChildItemsRaw">
							<xsl:for-each select="$leafPaths">
								<xsl:variable name="leafNodes" select="tokenize(.,'/')"/>
								<xsl:variable name="leafNodePath" select="concat($leafNodes[count($leafNodes)],'/')"/>
								<xsl:variable name="leafNodeParentPath" select="concat($leafNodes[count($leafNodes) - 1],'/')"/>
								<xsl:variable name="leafContextChildPath" select="concat($leafNodeParentPath,$leafNodePath)"/>
								<!-- xsl:message>leafContext: [<xsl:value-of select="$leafContextChildPath"/>]</xsl:message -->
								<xsl:variable name="leafParentContextItems" select="$dictionary//components/ITEMSlookup/Property[contains(@context,$leafNodePath)]"/>
								<!-- Now dedup this list -->
								<xsl:variable name="tempLeafParents">
									<xsl:for-each select="$leafParentContextItems">
										<list this="{@item}"/>
									</xsl:for-each>
								</xsl:variable>
								<xsl:variable name="uniqueLeafParentItems">
									<xsl:for-each select="distinct-values($tempLeafParents/list/@this)">
										<xsl:variable name="tempMatchLeafItem" select="."/>
										<xsl:copy-of select="$leafParentContextItems[@item=$tempMatchLeafItem][contains(@context,$parentPath)]"/><!-- match leaf with context path -->
									</xsl:for-each>
								</xsl:variable>

								<!-- $dictionary//components/ITEMSlookup/Property[contains(@context,$leafNodePath)] -->
								<xsl:for-each select="$uniqueLeafParentItems/Property">
								  	<!-- xsl:message>leafContextItem: [<xsl:value-of select="./@item"/>]</xsl:message -->
									<xsl:if test="not(contains(./@context,'@'))"><!-- process elements, not attributes -->
									<xsl:element name="Property">
										<xsl:for-each select=".//@*">
											<!-- xsl:if test="name()='context'"><xsl:message>attribute: [<xsl:value-of select="name()"/>] [<xsl:value-of select="."/>]</xsl:message></xsl:if -->
											<xsl:choose>
												<xsl:when test="name()='context'">
													<!-- adjust context path -->
													<xsl:variable name="tempPath" select="if(contains(.,$leafNodeParentPath)) then . else replace(concat($leafNodeParentPath,.),'//','/')"/>
													<xsl:attribute name="context" 
														select="if(starts-with($tempPath,'/')) then . else concat('/',$tempPath)"/>
												</xsl:when>
												<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
											</xsl:choose>	
										</xsl:for-each>
									</xsl:element>
									</xsl:if>
								   </xsl:for-each>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- xsl:message> children count: <xsl:value-of select="count($leafChildItemsRaw/Property)"/></xsl:message --><!-- child nodes in blueprint -->
						
						<xsl:variable name="uniqueLeafChildItems" select="distinct-values($leafChildItemsRaw/Property/concat(@item,'|',@context))"/>
						
						<!-- xsl:message> leaf children: <xsl:value-of select="$uniqueLeafChildItems"/></xsl:message -->
						
						<xsl:variable name="leafChildItems">
							<xsl:for-each select="$uniqueLeafChildItems">
								<xsl:variable name="thisChildItem" select="substring-before(.,'|')"/>
								<xsl:variable name="thisContextPath" select="substring-after(.,'|')"/>
								<xsl:variable name="tempChildItemsRaw" select="$leafChildItemsRaw/Property[@item=$thisChildItem][@context=$thisContextPath][1]"/>
								<xsl:copy-of select="$tempChildItemsRaw[1]"/>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- Doublecheck those child leaf items for additional children -->
						<xsl:variable name="additionalChildItems">
							<xsl:for-each select="$leafChildItems/Property">
								
								<xsl:variable name="contextNodes" select="tokenize(@context,'/')"/>
								<xsl:variable name="leafContextPathChk" select="concat($contextNodes[count($contextNodes) - 1],'/',@item, '/')"/>
								<xsl:variable name="addItemsList" select="$dictionary//components/ITEMSlookup/Property[contains(@context,$leafContextPathChk)]"/>
								<xsl:variable name="moreAddItems">
									<xsl:for-each select="$addItemsList">
										<list this="{@item}"/>
									</xsl:for-each>
								</xsl:variable>
								
								<xsl:for-each select="distinct-values($moreAddItems/list/@this)">
								     <xsl:variable name="moreAddItem" select="."/>
								     <xsl:for-each select="$dictionary//components/ITEMSlookup/Property[@item=$moreAddItem][contains(@context,$leafContextPathChk)][1]">
									<xsl:if test="not(contains(@context,'@'))"><!-- process elements, not attributes -->
									<xsl:variable name="thisItemName" select="@item"/>	
									<xsl:element name="Property">
										<xsl:for-each select=".//@*">
											<xsl:choose>
												<xsl:when test="name()='context'">
													<!-- adjust context path -->
													<xsl:variable name="tempPath" select="concat($leafContextPathChk,substring-after(.,$leafContextPathChk))"/>
													<xsl:attribute name="context" 
														select="if(starts-with($tempPath,'/')) then . else concat('/',$tempPath)"/>
													<!-- xsl:message> - - add child : [<xsl:value-of select="$thisItemName"/>] <xsl:value-of select="if(starts-with($tempPath,'/')) then . else concat('/',$tempPath)"/></xsl:message -->
												</xsl:when>
												<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
											</xsl:choose>	
										</xsl:for-each>
									</xsl:element>
									</xsl:if>
								     </xsl:for-each>
								</xsl:for-each>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- xsl:message>Additional: [<xsl:copy-of select="$additionalChildItems"/>]</xsl:message -->
						
						<!-- Now dedup these lists -->
						<xsl:variable name="uniqueLeafContext" select="distinct-values($leafChildItems/Property/@context)"/>
						<xsl:variable name="uniqueLeafChildItems">
							<xsl:for-each select="$uniqueLeafContext">
								<xsl:variable name="thisContext" select="."/>
								<xsl:copy-of select="$leafChildItems/Property[@context=$thisContext][1]"/>
							</xsl:for-each>
						</xsl:variable>
						
						<xsl:variable name="additionalLeafContext" select="distinct-values($additionalChildItems/Property/@context)"/>
						<xsl:variable name="combinedLeafChildItems">
							<xsl:copy-of select="$uniqueLeafChildItems"/>
							<xsl:for-each select="$additionalLeafContext">
								<xsl:variable name="thisContext" select="."/>
								<xsl:copy-of select="$additionalChildItems/Property[@context=$thisContext][1]"/>
							</xsl:for-each>
						</xsl:variable>
						
						<xsl:variable name="combinedLeafContext" select="distinct-values($combinedLeafChildItems/Property/@context)"/>
						<xsl:variable name="additionalLeafChildItems">
							<xsl:for-each select="$combinedLeafContext">
								<xsl:variable name="combinedContext" select="."/>
								<xsl:copy-of select="$combinedLeafChildItems/Property[@context=$combinedContext][1]"/>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- xsl:message>Leaves: [<xsl:copy-of select="$additionalLeafChildItems"/>]</xsl:message -->
						
						<xsl:variable name="importRest">
							<xsl:for-each select="$dedupStructure/Property">
								<xsl:choose>
									<xsl:when test="position()=1"/>
									<xsl:when test="@context=$contextMatch"><xsl:copy-of select="."/></xsl:when>
									<xsl:when test="contains(@context,concat($contextMatch,'/'))"><xsl:copy-of select="."/></xsl:when>
									<xsl:otherwise/>
								</xsl:choose>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- Now look to see if there are missing parent context items needed first, and add them if so -->
						
						<!-- Get the structure items path -->
						<xsl:variable name="contextPathItems" select="$dedupStructure/Property[1]/@context"/>
						<xsl:variable name="contextPathItem" select="$dedupStructure/Property[1]/@item"/>
						
						<xsl:variable name="recursionItems">
							<xsl:copy-of select="$dedupStructure/Property[1]"/>
						</xsl:variable>
						
						<xsl:choose>
							<xsl:when test="starts-with($contextPathItems,concat('/',$contextPathItem))">
								<!-- Structure members are all expanded so output -->
										<!-- xsl:message><xsl:value-of select="$dedupStructure/Property[1]/@item"/> all expanded import: [<xsl:value-of select="count($importRest//Property)"/>]</xsl:message -->
										<!-- xsl:message>more parent <xsl:value-of select="@item"/> - <xsl:value-of select="child::node()"/>[<xsl:value-of select="$thisItem"/>]</xsl:message -->
										<!-- xsl:message>Import D - <xsl:value-of select="$dedupStructure/Property[1]/@item"/></xsl:message -->
								<xsl:variable name="recursionList">
									<xsl:call-template name="recursionDetect">
										<xsl:with-param name="parentItem" select="$dedupStructure/Property[1]/@item"/>
										<xsl:with-param name="children" select="$dedupStructure/Property[1]/@children"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:apply-templates select="." mode="import">
									<xsl:with-param name="this" select="$dedupStructure/Property[1]"/>
									<xsl:with-param name="parent" select="'child::'"/>
									<xsl:with-param name="restItems" select="$importRest"/>
									<xsl:with-param name="childItems" select="$additionalLeafChildItems"/>
									<xsl:with-param name="recursionChk" select="$recursionList"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<!-- Expand output structure items -->
								<xsl:variable name="eachPathItem" select="tokenize($contextPathItems,'/')"/>
								
								<!-- Find parent items above this item -->
								<xsl:variable name="parentStructure">
									<xsl:for-each select="$eachPathItem">
										<xsl:variable name="pathPosition" select="position()"/>
										
										<xsl:if test="not(.=$contextPathItem) and(string-length(.) &gt; 0)">
											<xsl:variable name="thisPathItem" select="."/>
											<xsl:variable name="parentItemDetails" select="$dictionary//components/ITEMSlookup/Property[@item=$thisPathItem][1]"/>
											
											<!-- Insert Property for this path item -->
											<xsl:element name="Property">
												<xsl:for-each select="$parentItemDetails[1]//@*">
													<xsl:variable name="pathPrefix">
														<xsl:choose>
															<xsl:when test="string-length($eachPathItem[$pathPosition - 1]) &gt; 0">/<xsl:value-of select="$eachPathItem[$pathPosition - 1]"/></xsl:when>
															<xsl:otherwise/>
														</xsl:choose>
													</xsl:variable>
													<xsl:choose>
														<xsl:when test="name()='context'">
															<xsl:attribute name="context" select="concat($pathPrefix,'/',$thisPathItem,substring-after(.,$thisPathItem))"/>
														</xsl:when>
														<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
													</xsl:choose>	
												</xsl:for-each>
											</xsl:element>
										</xsl:if>
									</xsl:for-each>
								</xsl:variable>
								
								<xsl:variable name="combinedStructureItems">
									<xsl:for-each select="$parentStructure/Property">
										<xsl:choose>
											<xsl:when test="position()=1"/>
											<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
										</xsl:choose>
									</xsl:for-each>
									<xsl:copy-of select="$dedupStructure/Property[1]"/><!-- Match context here; right now just gets first one -->
									<xsl:copy-of select="$importRest"/>
								</xsl:variable>
								
								<!-- xsl:message>New structure: [<xsl:copy-of select="$combinedStructureItems"/>]</xsl:message -->
								
								<!-- Now output completed structure items -->
										<!-- xsl:message>Completed structure import</xsl:message -->
										<!-- xsl:message>complete parent <xsl:value-of select="$contextPathItem"/> - <xsl:copy-of select="$eachPathItem"/>[<xsl:value-of select="$thisItem"/>]</xsl:message -->
										<!-- xsl:message>Import E - <xsl:value-of select="$parentStructure/Property[1]/@item"/></xsl:message -->
										
								<xsl:variable name="recursionList">
									<xsl:call-template name="recursionDetect">
										<xsl:with-param name="parentItem" select="$parentStructure/Property[1]/@item"/>
										<xsl:with-param name="children" select="$parentStructure/Property[1]/@children"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:apply-templates select="." mode="import">
									<xsl:with-param name="this" select="$parentStructure/Property[1]"/>
									<xsl:with-param name="parent" select="$thisItem"/>
									<xsl:with-param name="restItems" select="$combinedStructureItems"/>
									<xsl:with-param name="childitems" select="$additionalLeafChildItems"/>
									<xsl:with-param name="recursionChk" select="$recursionList"/>
								</xsl:apply-templates>
							</xsl:otherwise>
						</xsl:choose>
																		
						<!-- Check for these property items, if not found add them to front of structure -->
						
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>structure here: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				<xsl:element name="{name()}" namespace="{$actualNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:if test="not(exists(*)) and string-length(text()) &gt; 0"><xsl:value-of select="text()"/></xsl:if><!-- insert default value text -->
					<xsl:if test="as:include">
						<xsl:apply-templates mode="include"/>
					</xsl:if>
					<xsl:message>- insert structure of: <xsl:value-of select="name()"/></xsl:message>
					<xsl:apply-templates mode="structure"/><!-- no select here - just insert the element and continue -->
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="text()" mode="structure"/>
	
	<xsl:template match="text()" mode="structureRules"/>
	
	<xsl:template match="@*" mode="insert">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:attribute name="{name()}" namespace="{$namespace-uri}"><xsl:value-of select="."/></xsl:attribute>
	</xsl:template>

	<xsl:template match="*|@*|text()">
	</xsl:template>

	<xsl:template match="*" mode="rule">
		<xsl:param name="action"/>

		<xsl:copy-of select="."/>
		
	</xsl:template>
	
	<!-- These templates are from original code -->
	
	<xsl:template match="as:constraint|as:context|@*" mode="startRules">
		<xsl:variable name="action" select="@action"/>
		<xsl:choose>
			<xsl:when test="contains($action,'applyTemplate')">
				<xsl:variable name="templateName" select="if (contains(substring-after($action,','),',')) then substring-before(substring-after($action,','),',') else substring-before(substring-after($action,','),')')"/>
				<xsl:variable name="templateFileName" select="concat('file://',$inputFilePath,'/',$templateName)"/>
				<!-- xsl:message>templateFileName=<xsl:value-of select="$templateFileName"/></xsl:message -->
				<xsl:variable name="template" select="document($templateFileName)"/>
				<xsl:variable name="applyTemplateRules" select="$template//as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint"/>
				<xsl:apply-templates select="$applyTemplateRules" mode="start"/>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*" mode="import">
		<xsl:param name="this"/>
		<xsl:param name="parent"/>
		<xsl:param name="restItems"/>
		<xsl:param name="childitems" as="node()" select="/"/>
		<xsl:param name="recursionChk"/>
		<xsl:variable name="thisItem" select="$this/./@item"/>
		<xsl:variable name="thisContext" select="$this/./@context"/>		
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="more" select="count($restItems/Property)"/>
		<xsl:variable name="matchPickParent" select="if(contains($parent,'/')) then concat('/',substring-before($parent,'/'),'/') else $parent"/>
		
		<xsl:variable name="thisPathContext">
			<xsl:choose>
				<xsl:when test="$parent = $thisItem"><xsl:value-of select="$parent"/></xsl:when>
				<xsl:otherwise><xsl:value-of  select="if(ends-with($parent,'/')) then concat($parent,$thisItem) else concat($parent,'/',$thisItem)"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:if test="not($recursionChk='')"><xsl:message> RECURSION: <xsl:value-of select="$thisItem"/> with <xsl:value-of select="tokenize($recursionChk,'&#xA;')"/></xsl:message></xsl:if>
		
		<xsl:variable name="parentNodesSeqn" select="tokenize($thisContext,'/')"/>
		<xsl:variable name="priorNodeCount" select="count($parentNodesSeqn) - 1"/>
		<xsl:variable name="chkParentSeqn">
			<xsl:choose>
				<xsl:when test="count($parentNodesSeqn) &lt; 3">true</xsl:when><!-- root node or singleton -->
				<xsl:when test="$parent=$parentNodesSeqn[$priorNodeCount]">true</xsl:when><!-- parent context must be immediately prior to this item -->
				<xsl:when test="$parent=$parentNodesSeqn[2]">true</xsl:when><!-- or parent context is the first node -->
				<xsl:when test="$parent='child::'">true</xsl:when><!-- or parent context is child -->
				<xsl:when test="contains($parent,'/') and(substring-before($parent,'/')=$parentNodesSeqn[$priorNodeCount])">true</xsl:when><!-- or parent context is the prior node -->
				<xsl:otherwise>false</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:if test="$debug &gt; 2"><xsl:message> importing - <xsl:value-of select="$thisItem"/></xsl:message></xsl:if>
		
		<!-- Now dedup restItems based on context -->
		<xsl:variable name="rest">
			<!-- Find the unique items to write out -->
			<xsl:variable name="uniqueItems" select="distinct-values($restItems/Property/concat(@item,'|',@context))"/>
			<xsl:variable name="allItemsNames">
				<xsl:for-each select="$uniqueItems"><xsl:value-of select="substring-before(.,'|')"/>|</xsl:for-each>
			</xsl:variable>
					<!-- xsl:message> uniques: <xsl:value-of select="distinct-values(tokenize($allItemsNames,'\|'))"/></xsl:message -->
					<!-- xsl:message> children: <xsl:value-of select="$this/@children"/></xsl:message -->
			<xsl:variable name="listA">
				<xsl:variable name="list" select="substring-after($this/@children,$nlchar)"/>
				<xsl:variable name="listItems" select="if(contains($list,'\')) then substring-before($list,'\') else $list"/>
				<xsl:for-each select="tokenize($listItems,$nlchar)"><xsl:if test="not(.='')"><xsl:value-of select="."/>|</xsl:if></xsl:for-each>
			</xsl:variable>
			<xsl:variable name="listB"><xsl:for-each select="distinct-values(tokenize($allItemsNames,'\|'))"><xsl:if test="not(.='')"><xsl:value-of select="."/>|</xsl:if></xsl:for-each></xsl:variable>
			<xsl:variable name="mergedItemsList" select="camed:merge-item-lists(concat(substring-before($listA,'|'),'|'),concat(substring-before($listB,'|'),'|') ,$listA,$listB,$listA)"/>
						<!-- xsl:message> merged: <xsl:value-of select="$mergedItemsList"/></xsl:message -->
						<xsl:if test="$debug &gt; 2"><xsl:message> - processing <xsl:value-of select="distinct-values(tokenize($mergedItemsList,'\|'))"/></xsl:message></xsl:if>
			<xsl:for-each select="distinct-values(tokenize($mergedItemsList,'\|'))">
			      <xsl:if test="not(.='')"><!-- process non-blank items -->
			      		
				<xsl:variable name="nextNameItem" select="."/>
			      		
				<xsl:variable name="matchedContextItems">
					<xsl:for-each select="$uniqueItems"><xsl:if test="starts-with(.,concat($nextNameItem,'|'))"><xsl:value-of select="."/>@</xsl:if></xsl:for-each>
				</xsl:variable>
			      	<xsl:variable name="completeContextItems">
			      		<xsl:choose>
			      			<xsl:when test="not($matchedContextItems ='')"><xsl:value-of select="$matchedContextItems"/></xsl:when>
			      			<xsl:otherwise><xsl:value-of select="concat(.,'|',$thisPathContext)"/>@</xsl:otherwise>			      				
			      		</xsl:choose>
			      	</xsl:variable>
			      		
				<xsl:variable name="bestMatchContextItems">
					<xsl:for-each select="max(tokenize($completeContextItems,'@'))"><xsl:value-of select="."/>@</xsl:for-each>
				</xsl:variable>
			      	
			      	<xsl:if test="$debug &gt; 1 and($matchedContextItems='')">
			      		<xsl:message> - unmatched: <xsl:value-of select="$nextNameItem"/> - <xsl:value-of select="$completeContextItems"/> [<xsl:value-of select="$bestMatchContextItems"/>]</xsl:message>
			      	</xsl:if>
				
				<!-- Pick the right items -->
				<xsl:for-each select="tokenize($bestMatchContextItems,'@')">
				   <xsl:variable name="thisRestItemName" select="substring-before(.,'|')"/>
				   <xsl:variable name="thisRestItemContext" select="substring-after(.,'|')"/>
				   <xsl:variable name="thisRestItems" select="$restItems/Property[@item=$thisRestItemName][@context=$thisRestItemContext]"/>
					<xsl:if test="$debug &gt; 3 and(not($thisRestItemName=''))"><xsl:message> - next <xsl:value-of select="$thisRestItemName"/></xsl:message></xsl:if>
				   <xsl:variable name="restPropertyCount" select="count($restItems/Property[@item=$thisRestItemName][@context=$thisRestItemContext])"/>
				   <xsl:choose>
				 	<xsl:when test="$thisRestItemName=''"/>
					<xsl:when test="$restPropertyCount &lt; 2 and($restPropertyCount &gt; 0)">
						<!-- Singleton item, so just process it -->
						<!-- xsl:message> singleton: <xsl:value-of select="$thisRestItemName"/> - <xsl:value-of select="$thisRestItemContext"/></xsl:message -->
						<xsl:copy-of select="$restItems/Property[@item=$thisRestItemName][@context=$thisRestItemContext]"/>
					</xsl:when>
				   	<xsl:when test="$restPropertyCount=0">
				   		<!-- Retrieve Property details for item not in restItems -->
				   		<xsl:variable name="retrieveContext" select="if(contains($thisRestItemContext,concat('child','::'))) then substring-after($thisRestItemContext,concat('child','::')) else $thisRestItemContext"/>
				   		<xsl:variable name="dictionaryProperty" select="$dictionary//components/ITEMSlookup/Property[@item=$thisRestItemName][contains(@context,$retrieveContext)]" />
				   		<xsl:if test="$debug &gt; 1"><xsl:message> - retrieving: <xsl:value-of select="$thisRestItemName"/> | <xsl:value-of select="$retrieveContext"/> - <xsl:value-of select="count($dictionaryProperty)"/></xsl:message></xsl:if>
				   		<xsl:copy-of select="$dictionaryProperty[1]"/>
				   	</xsl:when>
					<xsl:otherwise><!-- This only entered when multiples matching (short XPath BBIEs) -->
						<xsl:message> multiple context: <xsl:value-of select="$thisRestItemName"/> in <xsl:value-of select="$thisRestItemContext"/></xsl:message>
						<!-- Multiple items, so pick the best context to use: prefer 3 terms if available -->
						<xsl:variable name="theseItemsContext" select="$restItems/Property[@item=$thisRestItemName][@context=$thisRestItemContext]/@context"/>
						<xsl:variable name="contextLengths">
							<xsl:for-each select="$theseItemsContext">
								<!-- store term lengths for each context item -->
								<xsl:variable name="termsValue" select="tokenize(.,'/')"/>
								<length item="{$thisRestItemName}" value="{count($termsValue)}" context="{.}"/>
							</xsl:for-each>
						</xsl:variable>
						<!-- xsl:message>lengths: [<xsl:value-of select="$thisRestItemName"/>] [<xsl:value-of select="$contextLengths/length/@value"/>]</xsl:message -->
						<xsl:variable name="shortContextAvailable">
							<xsl:variable name="chkTerms" select="$contextLengths/length[@value &lt; 5]"/>
							<xsl:if test="count($chkTerms) &gt; 0">true</xsl:if>
						</xsl:variable>
						
						<xsl:variable name="chkThreeTerms" select="$contextLengths/length[@value = 4]/@context"/>
						
						<xsl:variable name="BestContextChoices">
							<xsl:choose>
								<xsl:when test="$contextLengths/length[@value = 4] and(count($restItems/Property[@item=$thisRestItemName][@context=$chkThreeTerms]) &gt; 0)"><xsl:value-of select="$contextLengths/length[@value = 4]/@context"/></xsl:when>
								<xsl:when test="$contextLengths/length[@value = 3]"><xsl:value-of select="$contextLengths/length[@value = 3]/@context"/></xsl:when>
								<xsl:when test="not($shortContextAvailable='true')"><xsl:value-of select="$contextLengths/length[@value = 5]/@context"/></xsl:when>
								<xsl:otherwise><xsl:value-of select="$contextLengths/length[@value &lt; 3]/@context"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						
						<xsl:variable name="eachContextChoice" select="distinct-values(tokenize($BestContextChoices,' '))"/>
						
						<xsl:variable name="pickBestContext">
							<xsl:choose>
								<xsl:when test="count($eachContextChoice) &lt; 3"><xsl:value-of select="$eachContextChoice"/></xsl:when>
								<xsl:otherwise>
									<xsl:for-each select="$eachContextChoice">
										<xsl:if test="contains(.,$matchPickParent)"><xsl:value-of select="."/></xsl:if>
									</xsl:for-each>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<xsl:variable name="pickBestOne" select="$restItems/Property[@item=$thisRestItemName][@context=$pickBestContext]"/>
						<xsl:copy-of select="$pickBestOne/Property[1]"/>
						<xsl:message>bestContext: <xsl:value-of select="$parent"/> / <xsl:value-of select="$thisRestItemName"/>[<xsl:value-of select="$eachContextChoice"/>][<xsl:value-of select="$pickBestContext"/>][<xsl:copy-of select="$pickBestOne/Property[1]"/>]</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
			     </xsl:for-each>
			   </xsl:if>
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:if test="$debug &gt; 2"><xsl:message> - queuing: <xsl:for-each select="$rest/Property"><xsl:value-of select="@item"/><xsl:if test="not(position() = count($rest/Property))">, </xsl:if></xsl:for-each></xsl:message></xsl:if>
		
		<xsl:choose>
			<xsl:when test="$chkParentSeqn='false'">
				<xsl:if test="$debug &gt; 1"><xsl:message> - skipped here: <xsl:value-of select="$thisItem"/> [<xsl:value-of select="$more"/>] - <xsl:value-of select="$parent"/> - [<xsl:value-of select="$parentNodesSeqn"/>] - [<xsl:value-of select="$parentNodesSeqn"/>]</xsl:message></xsl:if>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message>path: <xsl:value-of select="$thisItem"/> [<xsl:value-of select="$thisContext"/>] nodes: [<xsl:value-of select="$more"/>]</xsl:message>	
			</xsl:otherwise>
		</xsl:choose>
		
		<!-- xsl:if test="contains($thisItem,'TelephoneNumber') and($more &lt; 4)"><xsl:message>Rest: [<xsl:copy-of select="$rest"/>]</xsl:message></xsl:if -->
		<!-- xsl:if test="$thisContext='/lexs:UserAssertion/nc:ContactTelephoneNumber'"><xsl:message>Rest: [<xsl:copy-of select="$rest"/>]</xsl:message></xsl:if -->
		<!-- xsl:if test="contains($thisContext,'/lexs:DomainAttribute')"><xsl:message>here now: [<xsl:value-of select="$thisItem"/>][<xsl:copy-of select="$restItems"/>]</xsl:message></xsl:if -->
		
		<xsl:variable name="doubleChkSeqn">
			<xsl:choose><!-- Make sure the child items context reflect this parent context -->
				<xsl:when test="$more = 0">OK</xsl:when>
				<xsl:when test="$parent = $thisItem">OK</xsl:when>
				<xsl:when test="$parent = 'child::'">OK</xsl:when>
				<xsl:when test="contains($thisContext,concat('/',$parent[1],'/',$thisItem[1]))">OK</xsl:when>
				<xsl:when test="contains($parent,'/') and($rest/Property[contains(@context,$matchPickParent)])">OK</xsl:when>
				<!-- xsl:when test="$itemAnnotations//camed:annotation[@item=$thisContext]/camed:documentation[@type='Type']">OK</xsl:when --><!-- Type Reference in annotations -->
				<xsl:otherwise>IGNORE - <xsl:value-of select="concat('/',$parent[1],'/',$thisItem[1])"/>[<xsl:value-of select="$thisContext"/>]</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="$debug &gt; 0"><xsl:message> seqn chk: <xsl:value-of select="$chkParentSeqn"/> - <xsl:value-of select="$more"/> - <xsl:value-of select="$doubleChkSeqn"/> -  <xsl:value-of select="count($rest/Property)"/> -  <xsl:value-of select="$thisContext"/></xsl:message></xsl:if>
		
		<xsl:variable name="parentContext" select="concat($this/./@context,'/')"/>
		<xsl:choose>
			<xsl:when test="$this/./@CCTS ='ASBIE'"/><!-- ignore attributes -->
			<xsl:when test="$chkParentSeqn='false'"/><!-- ignore out of sequence items -->
			<xsl:when test="starts-with($doubleChkSeqn,'IGNORE -')"/><!-- ignore out of sequence items -->
			<xsl:when test="$more=0 and($this/./@CCTS ='ABIE') and(count($rest/Property)=0)"/><!-- ignore ABIE with no children -->
			<xsl:when test="$this/./@CCTS='ABIE' and($more &gt; 0 or($rest/Property))">
				<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="$thisItem"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:choose>
				 <xsl:when test="string-length($thisItem) &lt; 1">ERROR: context not matched to structure position [<xsl:copy-of select="$this"/>]</xsl:when>
				 <xsl:otherwise>
				 	<!-- xsl:message> - output element - <xsl:value-of select="$thisItem"/>- <xsl:value-of select="count($rest/Property)"/></xsl:message -->
				    <xsl:element name="{$thisItem}" namespace="{$actualNS}">
				       <xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
				       	<xsl:with-param name="mode">structure</xsl:with-param>
					<xsl:with-param name="item" select="$thisItem"/>
					<xsl:with-param name="contextPath" select="$thisContext"/>
				       </xsl:call-template>
				       <!-- xsl:if test="@item='im:Alien'"><xsl:message> Alien: <xsl:value-of select="count($rest//Property[@item='im:Alien'])"/></xsl:message></xsl:if -->	
				       <xsl:for-each select="$rest/Property">
					<!-- Now see if there are immediate children of this element -->
				       	<xsl:variable name="itemContextValue" select="@context"/>
					<xsl:variable name="chkContext" select="concat(@context,'/')"/>
					<xsl:variable name="itemContext" select="substring-after(@context,$parentContext)"/>
				  	<xsl:variable name="contextPath" select="concat($thisItem,'/',$itemContext)"/>
				  	<xsl:variable name="leafParentPath" select="substring-after($chkContext,$thisItem)"/>
				       	<xsl:variable name="pathItemDetails" select="tokenize(@context,'/')"/>
					<xsl:variable name="pathDepthCount" select="count($pathItemDetails)"/>
				       	
					<xsl:variable name="next-namespace-uri" select="namespace-uri()"/>
				       	<xsl:variable name="restContextValues" select="distinct-values($restItems/Property/@context)"/>
				       	
				       		<xsl:if test="$debug &gt; 0"><xsl:message> - rest item: <xsl:value-of select="$thisItem"/> / <xsl:value-of select="@item"/></xsl:message></xsl:if>
				       		<!-- xsl:message>Rest items context: [<xsl:value-of select="$chkContext"/>] [<xsl:copy-of select="$restContextValues"/>]</xsl:message -->
				  	<xsl:variable name="children">
				  		<xsl:choose>
				  			<xsl:when test="count($rest/Property[contains(@context,$chkContext)]) &gt; 0"><xsl:value-of select="count($rest/Property[contains(@context,$chkContext)])"/></xsl:when>
				  			<xsl:when test="$pathDepthCount &gt; 3"><!-- if full path does not match - try the sub-path against original restItems  -->
				  				<xsl:variable name="chkSubContext" select="substring-after($chkContext,concat($pathItemDetails[2],'/'))"/>
				  				<xsl:variable name="chkParentSubContext" select="concat('/',$chkSubContext)"/>
				  				<!-- xsl:message>chkSubContext: <xsl:value-of select="$chkSubContext"/> - <xsl:value-of select="$chkParentSubContext"/> - [<xsl:value-of select="count($restItems/Property)"/>] <xsl:value-of select="count($restItems/Property[contains(@context,$chkSubContext)])"/> - <xsl:value-of select="count($childitems/Property[contains(@context,$chkParentSubContext)][@CCTS='BBIE'][not(contains(@context,'@'))])"/> </xsl:message -->
				  				<xsl:choose>
				  					<xsl:when test="count($restItems/Property[contains(@context,$chkSubContext)]) &gt; 0"><xsl:value-of select="count($restItems/Property[contains(@context,$chkSubContext)])"/>|<xsl:value-of select="$chkSubContext"/></xsl:when>
				  					<xsl:otherwise><xsl:value-of select="0"/></xsl:otherwise>
				  				</xsl:choose>
				  			</xsl:when>
				  			<xsl:otherwise><xsl:value-of select="0"/></xsl:otherwise>
				  		</xsl:choose>
				  	</xsl:variable>
				       		<!-- xsl:message><xsl:value-of select="@item"/> - children: [<xsl:value-of select="$children"/>]</xsl:message -->
				       	<xsl:variable name="orphanPath">
				       		<xsl:for-each select="$pathItemDetails">
				       			<xsl:choose>
				       				<xsl:when test=".=''"/>
				       				<xsl:when test="position() &lt; 3"/>
				       				<xsl:otherwise><xsl:value-of select="concat('/',.)"/></xsl:otherwise>
				       			</xsl:choose>
				       		</xsl:for-each>
				       	</xsl:variable>
				       	
				       	<xsl:variable name="orphan" select="count($rest/Property[contains(@context,$orphanPath)])"/>
				       	
				       		<!-- xsl:message>orphan: [<xsl:value-of select="$orphan"/>][<xsl:value-of select="$orphanPath"/>]</xsl:message>
				       		<xsl:message>sub-context: [<xsl:value-of select="$thisItem"/>] depth [<xsl:value-of select="$pathDepthCount"/>] - [<xsl:value-of select="$chkContext"/>][<xsl:value-of select="$itemContext"/>][<xsl:value-of select="$children"/>][<xsl:value-of select="$chkParentSeqn"/>]</xsl:message -->
				       	
					<xsl:choose>
						<xsl:when test="$children &gt; '0' or($orphan &gt; 1)">
							<!-- Process child structure item -->
							
							<xsl:variable name="matchThisContext">
								<xsl:choose>
									<xsl:when test="contains($children,'|')"><xsl:value-of select="substring-after($children,'|')"/></xsl:when><!-- match from subContextPath -->
									<xsl:when test="$children &gt; 0"><xsl:value-of select="$chkContext"/></xsl:when><!-- direct match -->
									<xsl:otherwise><xsl:value-of select="$orphanPath"/></xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							
							<xsl:variable name="importRest">
								<xsl:for-each select="$rest/Property[contains(@context,$matchThisContext)]">
									<xsl:copy-of select="."/>
								</xsl:for-each>
							</xsl:variable>
													
							<!-- Continue with next structure items -->
							<xsl:variable name="recursionList">
								<xsl:call-template name="recursionDetect">
									<xsl:with-param name="parentItem" select="@item"/>
									<xsl:with-param name="children" select="@children"/>
								</xsl:call-template>
							</xsl:variable>
							<xsl:variable name="recursionTest" select="@item"/>
							<xsl:variable name="recursionCheck" select="concat($thisItem,'|',@item,'&#xA;')"/>
								<!-- xsl:message>this parent <xsl:value-of select="@item"/> - <xsl:value-of select="$parent"/>[<xsl:value-of select="$matchThisContext"/>][<xsl:value-of select="@context"/>]</xsl:message -->
								<!-- xsl:message>next part: [<xsl:value-of select="$matchThisContext"/>][<xsl:copy-of select="$importRest"/>]</xsl:message -->
								<!-- xsl:message>Import A - <xsl:value-of select="@item"/> - <xsl:value-of select="count($importRest//Property[@item=$recursionTest])"/></xsl:message -->
								<!-- xsl:message>recursive check: <xsl:value-of select="$recursionTest"/> - <xsl:value-of select="$recursionCheck"/></xsl:message -->
							<xsl:choose>
								<xsl:when test="contains($recursionChk,$recursionCheck)">
									<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="@item"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
									<xsl:element name="{@item}" namespace="{$actualNS}">recursive</xsl:element>
								</xsl:when>
								<xsl:when test="$recursionChk='' or(count($importRest//Property[@item=$recursionTest]) &lt; 1)">
									<xsl:apply-templates select="." mode="import">
										<xsl:with-param name="this" select="."/>
										<xsl:with-param name="parent" select="$thisItem"/> <!-- changed from thisItem -->
										<xsl:with-param name="restItems" select="$importRest"/>
										<xsl:with-param name="childitems" select="$childitems"/>
										<xsl:with-param name="recursionChk" select="concat(recursionChk,$recursionList)"/>
									</xsl:apply-templates>
								</xsl:when>
								<xsl:otherwise/><!-- Ignore other items -->
							</xsl:choose>
						</xsl:when>
						
						<!-- xsl:when test="contains($itemContext,'/x/')"  DISABLED --><!-- Ignore child of parent item --><!-- xsl:message>child ignored! <xsl:value-of select="$itemContext"/> - <xsl:value-of select="$parentContext"/> - <xsl:value-of select="@context"/></xsl:message></xsl:when -->
						<!-- xsl:when test="starts-with($chkContext,concat('/',$thisItem,'/')) and(count($pathItemDetails) &gt; 3)"/ --><!-- Ignore recursion back to start of path -->
						<xsl:when test="@CCTS ='ASBIE'"/><!-- ignore attributes -->
						<xsl:when test="$chkParentSeqn='false'"/><!-- ignore out of sequence items -->
						<xsl:otherwise>
							<!-- Process this singleton element -->
							<xsl:variable name="childPath" select="concat(@item,'/')"/>
									<!-- xsl:message>Childitems: <xsl:value-of select="$childPath"/>[<xsl:value-of select="count($childitems/Property[contains(@context,$childPath)])"/>]</xsl:message -->
							<xsl:choose>
								<xsl:when test="$childitems/Property[contains(@context,$childPath)]">
									<!-- Process leaves of this element node -->
									<xsl:variable name="theseChildNodes">
										<xsl:for-each select="$childitems/Property[contains(@context,$childPath)]">
											<xsl:copy-of select="."/>
										</xsl:for-each>
									</xsl:variable>
									<xsl:variable name="additionalChildNodes">
										<xsl:for-each select="$theseChildNodes/Property">
											<xsl:variable name="chkAdditionalNodes" select="tokenize(@context,'/')"/>
											<xsl:variable name="chkAdditionalPath" select="concat($chkAdditionalNodes[count($chkAdditionalNodes) - 1],'/',$chkAdditionalNodes[count($chkAdditionalNodes)])"/>
											
											<xsl:if test="not(starts-with($chkAdditionalPath,$childPath))">
												<!-- xsl:message>Additional leafs: [<xsl:value-of select="$childPath"/>] [<xsl:value-of select="$chkAdditionalPath"/>]</xsl:message -->
												<xsl:copy-of select="$childitems/Property[contains(@context,$chkAdditionalPath) and(not(contains(@context,$childPath)))]"/>
											</xsl:if>
										</xsl:for-each>
									</xsl:variable>
									
									<!-- Now dedup this list -->
									<xsl:variable name="additionalUniqueChildrenItems" select="distinct-values($additionalChildNodes/Property/@context)"/>
									<xsl:variable name="additionalUniqueChildren">
										<xsl:for-each select="$additionalUniqueChildrenItems">
											<xsl:variable name="thisContext" select="."/>
											<xsl:variable name="tempLeaf" select="$additionalChildNodes/Property[@context=$thisContext][1]"/>
											<xsl:element name="Property">
												<xsl:for-each select="$tempLeaf/@*">
													<xsl:attribute name="{name()}" select="."/>
												</xsl:for-each>
												<xsl:attribute name="leaf" select="'true'"/>
											</xsl:element>
										</xsl:for-each>
									</xsl:variable>
									
									<!-- Need to know what the actual structure path is to pick the right context path -->
									<xsl:variable name="thisLeafContextItem" select="@item"/>
									<xsl:variable name="thisLeafRoot" select="concat('/',$thisItem)"/>
									<xsl:variable name="thisLeafNode" select="concat('/',$thisLeafContextItem)"/>
									<xsl:variable name="thisLeafContext">
										<xsl:choose><!-- figure out from minefield, which parent path actually works here -->
											<xsl:when test="$childitems/Property[starts-with(@context,$thisLeafRoot)][ends-with(@context,$thisLeafNode)]/@context">
												<xsl:value-of select="$childitems/Property[starts-with(@context,$thisLeafRoot)][ends-with(@context,$thisLeafNode)]/@context"/>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="$childitems/Property[contains(@context,$thisLeafRoot)][ends-with(@context,$thisLeafNode)]/concat($thisLeafRoot,substring-after(@context,$thisLeafRoot))"/>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
									<!-- xsl:message>leaf context: <xsl:value-of select="$thisLeafContext"/></xsl:message -->
									<xsl:variable name="thisLeafContextNodes" select="tokenize($thisLeafContext[1],'/')"/>
									<xsl:variable name="thisLeafNodes" select="count($thisLeafContextNodes)"/>
									<xsl:variable name="thisLeafParent">
										<xsl:choose>
											<xsl:when test="$thisLeafNodes &lt; 4"><xsl:value-of select="$thisItem"/></xsl:when>
											<xsl:otherwise>
												<xsl:for-each select="$thisLeafContextNodes">
													<xsl:choose>
														<xsl:when test=".=''"/>
														<xsl:when test="position()=$thisLeafNodes"/>
														<xsl:when test=".=$thisLeafContextItem"/>
														<xsl:when test="position()=1"/>
														<xsl:otherwise><xsl:value-of select="concat('/',.)"/></xsl:otherwise>
													</xsl:choose>
												</xsl:for-each>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
									
										<!-- xsl:message>Leafs items: [<xsl:copy-of select="."/>][<xsl:copy-of select="$theseChildNodes"/>]</xsl:message -->
										<!-- xsl:message>leaf parent <xsl:value-of select="@item"/> - <xsl:value-of select="$parent"/>[<xsl:value-of select="$thisLeafParent"/>] - [<xsl:value-of select="$thisLeafNodes"/>]</xsl:message -->
										<!-- xsl:message>leaf property [<xsl:copy-of select="."/>][<xsl:copy-of select="$theseChildNodes"/>]</xsl:message -->
										<!-- xsl:message>Import B - <xsl:value-of select="@item"/></xsl:message -->
									<xsl:variable name="recursionList">
										<xsl:call-template name="recursionDetect">
											<xsl:with-param name="parentItem" select="@item"/>
											<xsl:with-param name="children" select="@children"/>
										</xsl:call-template>
									</xsl:variable>
									<xsl:apply-templates select="." mode="import">
										<xsl:with-param name="this" select="."/>
										<xsl:with-param name="parent" select="if(starts-with($thisLeafParent,'/')) then substring-after($thisLeafParent,'/') else $thisLeafParent"/>
										<xsl:with-param name="restItems" select="$theseChildNodes"/>
										<xsl:with-param name="childitems" select="if(count($theseChildNodes/Property)=0) then '/' else $additionalUniqueChildren"/>
										<xsl:with-param name="recursionChk" select="concat($recursionChk,$recursionList)"/>
									</xsl:apply-templates>
								</xsl:when>
								<xsl:when test="$chkParentSeqn='false'"/><!-- ignore out of sequence items -->
								<xsl:otherwise>
									<xsl:variable name="correctPath">
										<xsl:choose>
											<xsl:when test="$parent='child::'"><xsl:value-of select="concat('/',$thisItem,'/',@item)"/></xsl:when>
											<xsl:when test="$parent = $thisItem"><xsl:value-of select="concat($parent,'/',@item)"/></xsl:when>
											<xsl:when test="contains($parent,'/')"><xsl:value-of select="concat(substring-before($parent,'/'),'/',$thisItem,'/',@item)"/></xsl:when>
											<xsl:otherwise><xsl:value-of select="concat($parent,'/',$thisItem,'/',@item)"/></xsl:otherwise>
										</xsl:choose>
									</xsl:variable>
										<!-- xsl:message> - correct path: [<xsl:value-of select="@context"/>] context ends with: [<xsl:value-of select="$correctPath"/>]</xsl:message --> 
									<xsl:if test="ends-with(@context,$correctPath)"><!-- element is in correct context sequence, so write it out -->
										<!-- xsl:message> sub-path: <xsl:value-of select="@item"/> - <xsl:value-of select="$parent"/> - <xsl:value-of select="$correctPath"/> - - <xsl:value-of select="@context"/></xsl:message -->
										<xsl:message> item: <xsl:value-of select="@item"/></xsl:message>
										
										<xsl:variable name="shorterPath" select="substring-after($correctPath,'/')"/>
										<xsl:variable name="matchLookupPath">
											<xsl:choose>
												<xsl:when test="$dictionary//components/ITEMSlookup/Property[contains(@context,concat($correctPath,'/'))]"><xsl:value-of select="concat($correctPath,'/')"/></xsl:when>
												<xsl:when test="contains($shorterPath,'/') and $dictionary//components/ITEMSlookup/Property[contains(@context,concat(shorterPath,'/'))]"><xsl:value-of select="concat($shorterPath,'/')"/></xsl:when>
												<xsl:otherwise><xsl:value-of select="concat($correctPath,'/')"/><!-- xsl:message> picked path: <xsl:value-of select="concat($correctPath,'/')"/></xsl:message --></xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
											
										<!-- See if there are children, if so import those, otherwise, write this out -->
										<xsl:choose>
											<xsl:when test="$dictionary//components/ITEMSlookup/Property[contains(@context,$matchLookupPath)]">
												<xsl:variable name="matchParentNode" select="if(starts-with($matchLookupPath,concat($thisItem,'/'))) then $thisItem else $parent"/>
												<xsl:variable name="moreLeafNodeItems" select="$dictionary//components/ITEMSlookup/Property[contains(@context,$matchLookupPath)][contains(@context,concat($matchParentNode,'/'))]"/>
												<!-- Now dedup this list -->
												<xsl:variable name="tempLeafParents">
													<xsl:for-each select="$moreLeafNodeItems">
														<list this="{@item}"/>
													</xsl:for-each>
												</xsl:variable>
												<xsl:variable name="thisChildNodes">
													<xsl:for-each select="distinct-values($tempLeafParents/list/@this)">
														<xsl:variable name="tempMatchLeafItem" select="."/>
														<xsl:copy-of select="$moreLeafNodeItems[@item=$tempMatchLeafItem][1]"/>
													</xsl:for-each>
												</xsl:variable>
													<!-- xsl:message>child parent <xsl:value-of select="@item"/> - <xsl:value-of select="$parent"/> [<xsl:value-of select="$thisItem"/>] - <xsl:value-of select="$matchLookupPath"/></xsl:message -->
													<!-- xsl:message>Import C - <xsl:value-of select="@item"/></xsl:message -->
												<xsl:variable name="recursionList">
													<xsl:call-template name="recursionDetect">
														<xsl:with-param name="parentItem" select="@item"/>
														<xsl:with-param name="children" select="@children"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:choose>
													<xsl:when test="not($recursionList = '')"><!-- Node is recursive -->
														<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="@item"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
														<!-- xsl:element name="{@item}" namespace="{$actualNS}">recursive <xsl:value-of select="replace($recursionList,'&#xA;','  ')"/></xsl:element -->
														<xsl:element name="{@item}" namespace="{$actualNS}">
															<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																<xsl:with-param name="mode">structure</xsl:with-param>
																<xsl:with-param name="item" select="@item"/>
																<xsl:with-param name="contextPath" select="@context"/>
															</xsl:call-template>
															<xsl:for-each select="tokenize(@children,'&#xA;')">
																<xsl:variable name="child" select="."/>
																<xsl:if test="not($child='') and(position() &gt; 1)">
																	<xsl:choose>
																		<xsl:when test="contains($recursionList,concat($child,'|'))">																			
																			<xsl:element name="{$child}" namespace="{$actualNS}"><xsl:comment>recursive <xsl:value-of select="replace($recursionList,'&#xA;','  ')"/></xsl:comment><xsl:value-of select="'&#xA;'"></xsl:value-of>
																				<xsl:variable name="outputChildren" select="$dictionary//components/ITEMSlookup/Property[@item=$child]"/>
																				<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																					<xsl:with-param name="mode">structure</xsl:with-param>
																					<xsl:with-param name="item" select="$child"/>
																					<xsl:with-param name="contextPath" select="$outputChildren[1]/@context"/>
																				</xsl:call-template>
																				<xsl:for-each select="tokenize($outputChildren[1]/@children,'&#xA;')">
																					<xsl:if test="position() &gt; 1 and(. &gt; '') and(not(contains(.,'@')))">
																						<xsl:variable name="actualChildName" select="."/>
																						<xsl:variable name="actualChildNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="."/><xsl:with-param name="ns" select="$outputChildren[1]/namespace-uri()"/></xsl:call-template></xsl:variable>
																						<xsl:variable name="actualChild" select="$dictionary//components/ITEMSlookup/Property[@item=$actualChildName]"/>
																						<xsl:variable name="actualChildType"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$actualChild[1]/@type"/><xsl:with-param name="mask" select="$actualChild[1]/@mask"/></xsl:call-template></xsl:variable>
																						<xsl:element name="{.}" namespace="{$actualChildNS}">
																							<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																								<xsl:with-param name="mode">structure</xsl:with-param>
																								<xsl:with-param name="item" select="."/>
																								<xsl:with-param name="contextPath" select="$actualChild[1]/@context"/>
																							</xsl:call-template>
																							<xsl:choose>
																								<xsl:when test="contains($recursionList,concat(.,'|')) or(contains($recursionList,concat('|',.)))">recursive<xsl:comment>recursive entity</xsl:comment></xsl:when>
																								<xsl:when test="$actualChildType='' and($actualChild[1]/@CCTS = 'ABIE')"><!-- expand ABIE details -->
																									<xsl:for-each select="tokenize($actualChild[1]/@children,'&#xA;')">
																									     <xsl:variable name="nextChildName" select="."/>
																										<xsl:if test="position() &gt; 1 and($nextChildName &gt; '')  and(not(contains($nextChildName,'@')))">
																										<xsl:variable name="nextChildItem" select="$dictionary//components/ITEMSlookup/Property[@item=$nextChildName]"/>
																										<xsl:variable name="nextChildType"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$nextChildItem[1]/@type"/><xsl:with-param name="mask" select="$nextChildItem[1]/@mask"/></xsl:call-template></xsl:variable>
																										<xsl:variable name="nextChildNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="."/><xsl:with-param name="ns" select="$nextChildItem[1]/namespace-uri()"/></xsl:call-template></xsl:variable>
																									     	<xsl:choose>
																									     		<xsl:when test="not($nextChildItem[1]/@CCTS='ABIE')">
																									     			<xsl:element name="{.}" namespace="{$nextChildNS}">
																									     				<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																									     					<xsl:with-param name="mode">structure</xsl:with-param>
																									     					<xsl:with-param name="item" select="."/>
																									     					<xsl:with-param name="contextPath" select="$nextChildItem[1]/@context"/>
																									     				</xsl:call-template>
																									     				<xsl:value-of select="$nextChildType"/>
																									     			</xsl:element>
																									     		</xsl:when>
																									     		<xsl:when test="contains($recursionList,concat(.,'|')) or(contains($recursionList,concat('|',.)))">recursive<xsl:comment>recursive entity</xsl:comment></xsl:when>
																									     		<xsl:otherwise><!-- go one more level deep of expansion -->
																									     			<xsl:element name="{.}" namespace="{$nextChildNS}">
																									     				<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																									     					<xsl:with-param name="mode">structure</xsl:with-param>
																									     					<xsl:with-param name="item" select="."/>
																									     					<xsl:with-param name="contextPath" select="$nextChildItem[1]/@context"/>
																									     				</xsl:call-template>
																									     				<xsl:for-each select="tokenize($nextChildItem[1]/@children,'&#xA;')">
																									     					<xsl:variable name="lastChildName" select="."/>
																									     					<xsl:if test="position() &gt; 1 and($lastChildName &gt; '')  and(not(contains($lastChildName,'@')))">
																									     						<xsl:variable name="lastChildItem" select="$dictionary//components/ITEMSlookup/Property[@item=$lastChildName]"/>
																									     						<xsl:variable name="lastChildType"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$lastChildItem[1]/@type"/><xsl:with-param name="mask" select="$lastChildItem[1]/@mask"/></xsl:call-template></xsl:variable>
																									     						<xsl:variable name="lastChildNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="."/><xsl:with-param name="ns" select="$lastChildItem[1]/namespace-uri()"/></xsl:call-template></xsl:variable>
																									     						<xsl:element name="{.}" namespace="{$lastChildNS}">
																									     							<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																									     								<xsl:with-param name="mode">structure</xsl:with-param>
																									     								<xsl:with-param name="item" select="."/>
																									     								<xsl:with-param name="contextPath" select="$lastChildItem[1]/@context"/>
																									     							</xsl:call-template>
																									     							<xsl:value-of select="$lastChildType"/>
																									     						</xsl:element>
																									     					</xsl:if>
																									     				</xsl:for-each>
																									     			</xsl:element>
																									     		</xsl:otherwise>
																									     	</xsl:choose>																									     	
																									     	
																									    </xsl:if>
																									</xsl:for-each>
																								</xsl:when>
																								<xsl:otherwise><xsl:value-of select="$actualChildType"/></xsl:otherwise>
																							</xsl:choose>
																						</xsl:element>
																					</xsl:if>
																				</xsl:for-each>
																			</xsl:element>
																		</xsl:when>
																		<xsl:otherwise>
																			<!-- lookup the item and expand it -->
																			<xsl:variable name="thisChildItem" select="$dictionary//components/ITEMSlookup/Property[@item=$child]"/>
																			<xsl:variable name="actualChildNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="."/><xsl:with-param name="ns" select="$thisChildItem[1]/namespace-uri()"/></xsl:call-template></xsl:variable>
																			<xsl:variable name="actualChildType"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$thisChildItem[1]/@type"/><xsl:with-param name="mask" select="$thisChildItem[1]/@mask"/></xsl:call-template></xsl:variable>
																			<xsl:variable name="thisChildItems"><Property/></xsl:variable>
																			<xsl:element name="{$child}" namespace="{$actualChildNS}">
																				<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
																					<xsl:with-param name="mode">structure</xsl:with-param>
																					<xsl:with-param name="item" select="$child"/>
																					<xsl:with-param name="contextPath" select="$thisChildItem[1]/@context"/>
																				</xsl:call-template>
																				<xsl:value-of select="$actualChildType"/>
																			</xsl:element>
																			<xsl:apply-templates select="$thisChildItem[1]" mode="import"><!-- output other nodes (empty children) -->
																				<xsl:with-param name="this" select="$thisChildItem[1]"/>
																				<xsl:with-param name="parent" select="$matchParentNode"/>
																				<xsl:with-param name="restItems" select="$thisChildItems"/>
																				<xsl:with-param name="childitems" select="$thisChildItems"/>
																				<xsl:with-param name="recursionChk" select="concat($recursionChk,$recursionList)"/>
																			</xsl:apply-templates>
																		</xsl:otherwise>
																	</xsl:choose>
																</xsl:if>
															</xsl:for-each>
														</xsl:element>
													</xsl:when>
													<xsl:otherwise>
														<xsl:apply-templates select="." mode="import">
															<xsl:with-param name="this" select="."/>
															<xsl:with-param name="parent" select="$matchParentNode"/>
															<xsl:with-param name="restItems" select="$thisChildNodes"/>
															<xsl:with-param name="childitems" select="$childitems"/>
															<xsl:with-param name="recursionChk" select="concat($recursionChk,$recursionList)"/>
														</xsl:apply-templates>
													</xsl:otherwise>
												</xsl:choose>
											</xsl:when>
											<xsl:otherwise><!-- Write out this singleton node -->
												<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="@item"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
												<!-- xsl:message>  singleton A: <xsl:value-of select="@item"/> [<xsl:value-of select="$thisItem"/>]</xsl:message -->
												<xsl:element name="{@item}" namespace="{$actualNS}" >
													<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
														<xsl:with-param name="mode">structure</xsl:with-param>
														<xsl:with-param name="item" select="@item"/>
														<xsl:with-param name="contextPath" select="@context"/>
													</xsl:call-template>	
													
													<xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="@type"/><xsl:with-param name="mask" select="@mask"/></xsl:call-template>
												</xsl:element>
											</xsl:otherwise>
										</xsl:choose>			
									</xsl:if>
									<!-- If this is in fact a parent node with no matching children in restItems, then output type example -->
									<xsl:if test="not(ends-with(@context,$correctPath)) and(concat($this/./@type,$this/./@mask) &gt; '')">
										<!-- xsl:message> - set type  [<xsl:value-of select="$thisItem"/>] [<xsl:value-of select="$this/./@type"/>] [<xsl:value-of select="$this/./@mask"/>]</xsl:message -->
										<xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$this/./@type"/><xsl:with-param name="mask" select="$this/./@mask"/></xsl:call-template>
									</xsl:if>
								</xsl:otherwise>
							</xsl:choose>			
						</xsl:otherwise>
					</xsl:choose>
				    </xsl:for-each>
				   <!-- never comes in here - aka there are always child elements -->     
				    </xsl:element>
				 </xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<!-- Process this singleton element -->
				<xsl:if test="string-length($thisItem) &gt; 0">
					<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="$thisItem"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
					<!-- xsl:message>  singleton B: <xsl:value-of select="$thisItem"/></xsl:message -->
					<xsl:element name="{$thisItem}" namespace="{$actualNS}">
						<xsl:call-template name="lookupAttrs"><!-- Lookup if any attributes for this item -->
							<xsl:with-param name="mode">structure</xsl:with-param>
							<xsl:with-param name="item" select="$thisItem"/>
							<xsl:with-param name="contextPath" select="$thisContext"/>
						</xsl:call-template>
						<xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="$this/./@type"/><xsl:with-param name="mask" select="$this/./@mask"/></xsl:call-template>
					</xsl:element>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*" mode="structureRules">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		<xsl:variable name="parentChildPath" select="concat('/',$parent,'/',$thisItem)"/>
		
		<!-- Find the names of the child items of this node -->
		<xsl:variable name="theseChildrenMembers" select="child::node()"/>
		<xsl:variable name="ChildElementNames">
			<xsl:for-each select="$theseChildrenMembers/.">|<xsl:value-of select="name()"/>|</xsl:for-each>
		</xsl:variable>
		
		<xsl:variable name="childNodesCount" select="count(tokenize($ChildElementNames,'\|\|'))"/>
		
		<!-- xsl:if test="not($ChildElementNames='||')"><xsl:message>child nodes: [<xsl:value-of select="$childNodesCount"/>] <xsl:value-of select="concat($parent,$contextMatch)"/> - <xsl:value-of select="$ChildElementNames"/></xsl:message></xsl:if -->

		<!-- Choose to lookup item structure in dictionary, or simply skip  -->
		<!-- xsl:if test="contains($parent,'Attachment')"><xsl:message> - <xsl:value-of select="$parentChildPath"/> | <xsl:value-of select="$contextMatch"/> | <xsl:value-of select="$thisNS"/></xsl:message></xsl:if -->		
		<xsl:choose>
			<xsl:when test="contains($namespacesList,$thisNS)">
				<!-- Lookup in dictionary -->
				<xsl:variable name="importStructureRules">
					<xsl:choose>
						<!-- ignore items that are flagged as already expanded -->
						<xsl:when test="$expansionItems/expander/camed:annotation[ends-with(@item,$contextMatch)]"><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:when><!-- ignore items already expanded -->
						<xsl:when test="$dictionary//components/ITEMSlookup/Property[@item=$thisItem]">
							<!-- Find matching items -->
							
							<xsl:choose>
								<xsl:when test="count($dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]) &gt; 0">
									<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@context = $parentChildPath]"/><!-- Get the parent node property -->
									<xsl:for-each select="$dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]">
										
										<xsl:choose>
											<xsl:when test="not($ChildElementNames='||') "><!-- check items from dictionary match nodes that are in the structure output -->
												<!-- xsl:message> rule: <xsl:value-of select="@item"/> - <xsl:value-of select="@context"/></xsl:message -->
												<xsl:choose>
													<xsl:when test="contains($ChildElementNames,concat('|',@item,'|'))">
														<Property>
															<xsl:for-each select="@*">
																<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
															</xsl:for-each>
														</Property>
													</xsl:when>
													<xsl:otherwise>
														<xsl:variable name="findNodeMatch">
														<xsl:for-each select="tokenize(@context,'/')">
															<xsl:if test="string-length(.) &gt; 1 and(contains($ChildElementNames,concat('|',.,'|')))">true</xsl:if>
														</xsl:for-each>
														</xsl:variable>
														<xsl:if test="contains($findNodeMatch,'true')">
															<Property>
																<xsl:for-each select="@*">
																	<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
																</xsl:for-each>
															</Property>
														</xsl:if>
													</xsl:otherwise>
												</xsl:choose>
											</xsl:when>
											<xsl:otherwise>
												<Property>
													<xsl:for-each select="@*">
														<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
													</xsl:for-each>
												</Property>
											</xsl:otherwise>
										</xsl:choose>
										
									</xsl:for-each>
									<!-- xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]"/ --><!-- rules selection  -->
								</xsl:when>
								<xsl:otherwise><!-- its a child item -->
									<xsl:variable name="childPathMatch" select="concat($contextMatch,'/')"/>
									<xsl:variable name="tempDefinition">
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem][1]"/>
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[contains(@context,$childPathMatch)]"/>
									</xsl:variable>
									<xsl:element name="Property">
										<xsl:for-each select="$tempDefinition//@*">
											<xsl:choose>
												<xsl:when test="name()='context'">
													<xsl:attribute name="context" select="concat('/',$thisItem,substring-after(.,$thisItem))"/>
												</xsl:when>
												<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
											</xsl:choose>	
										</xsl:for-each>
									</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<!-- xsl:if test="contains($parentChildPath,'/lexs:AttachmentResponseMessage/lexs:Attachment')"><xsl:message>rules: [<xsl:copy-of select="$importStructureRules"/>]</xsl:message></xsl:if -->
				<xsl:choose>
					<xsl:when test="$importStructureRules/no_match"><xsl:apply-templates mode="structureRules"/></xsl:when><!-- Skip to next structure item -->
					<xsl:otherwise><!-- Output structure rule details from dictionary definition -->
						
						<xsl:variable name="importRest">
							<xsl:for-each select="$importStructureRules/Property">
								<!-- xsl:if test="contains(@item,'Attachment')"><xsl:message> - <xsl:value-of select="@item"/> - <xsl:value-of select="position()"></xsl:value-of></xsl:message></xsl:if -->
								<xsl:choose>
									<xsl:when test="position()=1"/>
									<xsl:when test="@context=$contextMatch"><xsl:copy-of select="."/></xsl:when>
									<xsl:when test="@context=$parentChildPath"><xsl:copy-of select="."/></xsl:when>
									<xsl:when test="contains(@context,concat($contextMatch,'/'))"><xsl:copy-of select="."/></xsl:when>
									<xsl:otherwise/>
								</xsl:choose>
							</xsl:for-each>
						</xsl:variable>
						<!-- dedup the list -->
						<xsl:variable name="uniqueImportItems" select="distinct-values($importRest/Property/@context)"/>
						<xsl:variable name="uniqueRestImport">
							<xsl:for-each select="$uniqueImportItems">
								<xsl:variable name="thisContext" select="."/>
								<xsl:variable name="tempItem" select="$importRest/Property[@context=$thisContext][1]"/>
								<xsl:element name="Property">
									<xsl:for-each select="$tempItem/@*">
										<xsl:attribute name="{name()}" select="."/>
									</xsl:for-each>
								</xsl:element>
							</xsl:for-each>
						</xsl:variable>
						
						<!-- Output dictionary rule details for this item -->
						<xsl:if test="count($importStructureRules/Property[@item=$thisItem]) = 0">
							<xsl:variable name="thisItemRules" select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem][1]"/>
							<xsl:if test="$debug &gt; 0"><xsl:message> rules - <xsl:value-of select="$thisItem"/> - <xsl:value-of select="$thisItemRules/@CCTS"/></xsl:message></xsl:if>
							<!-- Cardinality rules -->
							<xsl:if test="contains($thisItemRules[1]/@cardinality,'optional')">
								<as:constraint action="{concat('makeOptional(/',$parentChildPath,')')}"/>
							</xsl:if>
							<xsl:if test="contains($thisItemRules[1]/@cardinality,'repeatable')">
								<as:constraint action="{concat('makeRepeatable(/',$parentChildPath,')')}"/>
							</xsl:if>
							<!-- Data type rules -->
							<xsl:if test="$thisItemRules[1]/@CCTS='BBIE'">
								<xsl:call-template name="setFacetRules">
									<xsl:with-param name="type" select="$thisItemRules/@type"/>
									<xsl:with-param name="mask" select="$thisItemRules/@mask"/>
									<xsl:with-param name="path" select="concat('/',$parentChildPath)"/>
								</xsl:call-template>
							</xsl:if>
						</xsl:if>
						<!-- xsl:message>Dedup: [<xsl:copy-of select="$uniqueImportItems"/>][<xsl:copy-of select="$uniqueRestImport"/>]</xsl:message -->
						<xsl:apply-templates select="." mode="importRules">
							<xsl:with-param name="this" select="$importStructureRules/Property[1]"/>
							<xsl:with-param name="parent" select="$parent"/>
							<xsl:with-param name="rest" select="$uniqueRestImport"/>
						</xsl:apply-templates>
						<xsl:apply-templates mode="structureRules"/>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:if test="attribute::node()">
					<!-- xsl:message>ATTRIBUTE! <xsl:value-of select="$thisItem"/><xsl:for-each select="@*">@<xsl:value-of select="name()"/></xsl:for-each></xsl:message -->
					<xsl:apply-templates select="@*" mode="structureRules">
						<xsl:with-param name="context" select="concat($contextMatch,'@')"/>
					</xsl:apply-templates>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise>				
				<xsl:apply-templates mode="structureRules"/>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="@*" mode="structureRules">
		<xsl:param name="context"/>
		<xsl:variable name="findAttrPath" select="concat($context,name())"/>
		<xsl:for-each select="$dictionary//components/ITEMSlookup/Property[ends-with(@context,$findAttrPath)][1]">
			<xsl:if test="contains(./@cardinality,'optional')">
				<as:constraint action="{concat('makeOptional(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:if test="contains(./@cardinality,'repeatable')">
				<as:constraint action="{concat('makeRepeatable(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:call-template name="setFacetRules">
				<xsl:with-param name="type" select="@type"/>
				<xsl:with-param name="mask" select="@mask"/>
				<xsl:with-param name="path" select="concat('/',@context)"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="*" mode="importRules">
		<xsl:param name="this"/>
		<xsl:param name="parent"/>
		<xsl:param name="rest"/>
		<xsl:variable name="thisItem" select="$this/./@item"/>
		<xsl:variable name="thisContext" select="$this/./@context"/>		
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="more" select="count($rest/Property)"/>
		<xsl:variable name="parentContext" select="concat($this/./@context,'/')"/>
		<xsl:variable name="treeNode" select="../../name()"/>
		<xsl:variable name="priorParent" select="../name()"/>
		
		<xsl:variable name="fullContextPath">
			<xsl:choose>
				<xsl:when test="not($treeNode='')"><xsl:value-of select="concat('/',$treeNode,'/',$parent,'/',$thisItem)"/></xsl:when>
				<xsl:when test="not($priorParent='')"><xsl:value-of select="concat('/',$priorParent,'/',$thisItem)"/></xsl:when>
				<xsl:when test="not($parent='')"><xsl:value-of select="concat('/',$parent,'/',$thisItem)"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('/',$thisItem)"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:if test="$debug &gt; 0"><xsl:message>  rules <xsl:value-of select="$thisItem"/> - [<xsl:value-of select="$fullContextPath"/>][<xsl:value-of select="$more"/>]</xsl:message></xsl:if>
		<!-- xsl:if test="ends-with($fullContextPath ,'lexs:Attachment')"><xsl:message> - <xsl:copy-of select="$rest"/></xsl:message></xsl:if -->
		<xsl:choose>
			<!-- xsl:when test="$this/./@CCTS ='ASBIE'"/ --><!-- ignore attributes -->
			<xsl:when test="$more &gt; 0">
				
				<xsl:if test="contains($this/./@cardinality,'optional')">
					<as:constraint action="{concat('makeOptional(/',$thisContext,')')}"/>
				</xsl:if>
				<xsl:if test="contains(@cardinality,'repeatable')">
					<as:constraint action="{concat('makeRepeatable(/',$thisContext,')')}"/>
				</xsl:if>
				
				<xsl:for-each select="$rest/Property">
					<!-- xsl:message>rest: [<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$parentContext"/>][<xsl:value-of select="@context"/>]</xsl:message -->
					<!-- Now see if there are children of this element -->
					<xsl:variable name="chkContext" select="concat(@context,'/')"/>
					<xsl:variable name="itemContext" select="substring-after(@context,$parentContext)"/>
					<xsl:variable name="next-namespace-uri" select="namespace-uri()"/>
					<xsl:variable name="children" select="count($rest/Property[starts-with(@context,$chkContext)])"/>

					<xsl:choose>
						<xsl:when test="$children &gt; 0">
							<!-- Process parent and children -->
							<xsl:variable name="importRest">
								<xsl:for-each select="$rest/Property[contains(@context,$chkContext)]">
									<xsl:copy-of select="."/>
								</xsl:for-each>
							</xsl:variable>
							
							<!-- dedup list -->
							<xsl:variable name="uniqueImportItems" select="distinct-values($importRest/Property/@context)"/>
							<xsl:variable name="uniqueRestImport">
								<xsl:for-each select="$uniqueImportItems">
									<xsl:variable name="thisContext" select="."/>
									<xsl:variable name="tempItem" select="$importRest/Property[@context=$thisContext][1]"/>
									<xsl:element name="Property">
										<xsl:for-each select="$tempItem/@*">
											<xsl:attribute name="{name()}" select="."/>
										</xsl:for-each>
									</xsl:element>
								</xsl:for-each>
							</xsl:variable>
							
							<!-- xsl:message>Dedup 1: [<xsl:copy-of select="$uniqueImportItems"/>][<xsl:copy-of select="$uniqueRestImport"/>]</xsl:message -->
							<xsl:apply-templates select="." mode="importRules">
								<xsl:with-param name="this" select="."/>
								<xsl:with-param name="parent" select="$thisItem"/>
								<xsl:with-param name="rest" select="$uniqueRestImport"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="contains($itemContext,'/')"><!-- Ignore child of parent item --></xsl:when>
						<xsl:otherwise><!-- Process this singleton element -->
							
							<!-- Adjust path to full node path -->
							<xsl:variable name="thisContextNodes" select="tokenize(@context,'/')"/>
							<xsl:variable name="fullNodePath">
								<xsl:choose>
									<xsl:when test="count($thisContextNodes) &lt; 4 and(string-length($priorParent) &gt; 0)">
										<xsl:value-of select="concat('/',$priorParent,@context)"/>
									</xsl:when>
									<xsl:otherwise><xsl:value-of select="@context"/></xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							<!-- xsl:message>rule: <xsl:value-of select="$fullNodePath"/> - <xsl:value-of select="count($thisContextNodes)"/></xsl:message -->
							<xsl:if test="contains(./@cardinality,'optional')">
								<as:constraint action="{concat('makeOptional(/',$fullNodePath,')')}"/>
							</xsl:if>
							<xsl:if test="contains(./@cardinality,'repeatable')">
								<as:constraint action="{concat('makeRepeatable(/',$fullNodePath,')')}"/>
							</xsl:if>
							<xsl:call-template name="setFacetRules">
								<xsl:with-param name="type" select="@type"/>
								<xsl:with-param name="mask" select="@mask"/>
								<xsl:with-param name="path" select="concat('/',$fullNodePath)"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="matchItem" select="$this/./@item"/>
				<xsl:variable name="dictionaryContext" select="$dictionary//components/ITEMSlookup/Property[@item=$matchItem][contains(@context,$parent)]/@context"/>
				
				<!-- Adjust path to full node path -->
				<xsl:variable name="thisContextNodes" select="tokenize($this/./@context,'/')"/>
				<xsl:variable name="fullNodePath">
					<xsl:choose>
						<xsl:when test="count($thisContextNodes) &lt; 4 and(string-length($priorParent) &gt; 0)">
							<xsl:value-of select="concat('/',$priorParent,$this/./@context)"/>
						</xsl:when>
						<xsl:otherwise><xsl:value-of select="$this/./@context"/></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				
				<xsl:variable name="contextDepth">
					<xsl:for-each select="$dictionaryContext">
						<xsl:variable name="thisContext" select="tokenize(.,'/')"/>
						<xsl:for-each select="$thisContext">
							<xsl:if test=".=$matchItem"><context value="{position()}"/></xsl:if>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="leastContextDepth" select="count($contextDepth/context[@value &gt; 2])"/>
				
				<!-- xsl:message>Least: <xsl:value-of select="$matchItem"/> - <xsl:value-of select="$parent"/> -<xsl:value-of select="$leastContextDepth"/></xsl:message -->
				
				<xsl:choose>
				  <xsl:when test="$leastContextDepth = 0 and(count($contextDepth/context) &gt; 1 or(count($contextDepth/context) = 0))">
				  	<!-- Process singleton that is not actually a child leaf -->
				  	
					<xsl:if test="contains($this/./@cardinality,'optional')">
						<as:constraint action="{concat('makeOptional(/',$fullContextPath,')')}"/>
					</xsl:if>
					<xsl:if test="contains(@cardinality,'repeatable')">
						<as:constraint action="{concat('makeRepeatable(/',$fullContextPath,')')}"/>
					</xsl:if>
					<xsl:call-template name="setFacetRules">
						<xsl:with-param name="type" select="$this/./@type"/>
						<xsl:with-param name="mask" select="$this/./@mask"/>
						<xsl:with-param name="path" select="concat('/',$fullContextPath)"/>
					</xsl:call-template>
				  </xsl:when>
				  <xsl:when test="count(tokenize($fullContextPath,'/')) &gt; 3">	
					<xsl:if test="contains($this/./@cardinality,'optional')">
						<as:constraint action="{concat('makeOptional(/',$fullContextPath,')')}"/>
					</xsl:if>
					<xsl:if test="contains(@cardinality,'repeatable')">
						<as:constraint action="{concat('makeRepeatable(/',$fullContextPath,')')}"/>
					</xsl:if>
					<xsl:call-template name="setFacetRules">
						<xsl:with-param name="type" select="$this/./@type"/>
						<xsl:with-param name="mask" select="$this/./@mask"/>
						<xsl:with-param name="path" select="concat('/',$fullContextPath)"/>
					</xsl:call-template>
				  </xsl:when>
				  <xsl:otherwise/>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="setFacetRules">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:param name="path"/>
		<xsl:variable name="thisPath" select="if(contains($path,'/@')) then $path else replace($path,'@','/@')"/>
		<xsl:if test="$debug &gt; 0"><xsl:message> - <xsl:value-of select="$type"/> - <xsl:value-of select="$mask"/> [<xsl:value-of select="$path"/>]</xsl:message></xsl:if>
		<xsl:choose>
			<xsl:when test="$type='ID'"><as:constraint action="{concat('datatype(',$thisPath,',ID)')}"/></xsl:when>
			<xsl:when test="$type='IDREF'"><as:constraint action="{concat('datatype(',$thisPath,',IDREF)')}"/></xsl:when>
			<xsl:when test="$type='IDREFS'"><as:constraint action="{concat('datatype(',$thisPath,',IDREFS)')}"/></xsl:when>
			<xsl:when test="$type='any'"><as:constraint action="{concat('datatype(',$thisPath,',any)')}"/></xsl:when>
			<xsl:when test="$type='anyURI'"><as:constraint action="{concat('datatype(',$thisPath,',anyURI)')}"/></xsl:when>
			<xsl:when test="$type='base64Binary'"><as:constraint action="{concat('datatype(',$thisPath,',base64Binary)')}"/></xsl:when>
			<xsl:when test="$type='anyAttribute'"><as:constraint action="{concat('datatype(',$thisPath,',anyAttribute)')}"/></xsl:when>
			<xsl:when test="$type='boolean'"><as:constraint action="{concat('datatype(',$thisPath,',boolean)')}"/></xsl:when>
			<xsl:when test="$type='token'"><as:constraint action="{concat('datatype(',$thisPath,',token)')}"/></xsl:when>
			<xsl:when test="$type='language'"><as:constraint action="{concat('datatype(',$thisPath,',language)')}"/></xsl:when>
			<xsl:when test="contains($restrictionRules,'None')"/>
			<xsl:when test="$type='integer' and($useNumbers = 'true')"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='integer' and($useNumbers = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',integer)')}"/></xsl:when>
			<xsl:when test="$type='decimal' and($useNumbers = 'true')"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='decimal' and($useNumbers = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',decimal)')}"/></xsl:when>
			<xsl:when test="$type='float' and($useNumbers = 'true')"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='float' and($useNumbers = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',float)')}"/></xsl:when>
			<xsl:when test="$type='unsignedInt' and($useNumbers = 'true')"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask[1]='') then '########' else $mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='unsignedInt' and($useNumbers = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',unsignedInt)')}"/></xsl:when>
			<xsl:when test="$type='unsignedLong' and($useNumbers = 'true')"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask[1]='') then '############' else $mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='unsignedLong' and($useNumbers = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',unsignedLong)')}"/></xsl:when>
			<xsl:when test="$type='date' and($useDates = 'true')"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='date' and($useDates = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',date)')}"/></xsl:when>
			<xsl:when test="$type='time' and($useDates = 'true')"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='time' and($useDates = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',time)')}"/></xsl:when>
			<xsl:when test="$type='datetime' and($useDates = 'true')"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask[1],')')}"/></xsl:when>
			<xsl:when test="$type='datetime' and($useDates = 'false')"><as:constraint action="{concat('datatype(',$thisPath,',dateTime)')}"/></xsl:when>
			<xsl:when test="$type='list' and($useCodes = 'true')"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask[1],',','''|'''),''')')}"/></xsl:when>
			<xsl:when test="contains($mask[1],'|') and($useCodes = 'true')"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask[1],'\|','''|'''),''')')}"/></xsl:when>
			<xsl:when test="$type='string' and(not($mask[1]='')) and($useStrings = 'true')">
				<xsl:variable name="chkNamePath" select="reverse(tokenize($thisPath,'/'))"/>
				<xsl:variable name="chkName" select="upper-case($chkNamePath[1])"/>
				<xsl:variable name="maxLength" select="xsd:integer(substring-after($mask[1],'-'))"/>
				<xsl:variable name="assignMask">
				<xsl:choose><!-- These rules are designed to ensure length values are logical for business rules.  To override use "All" in restrictionRules -->
					<xsl:when test="ends-with($chkName,'CODE') and($maxLength &lt; 41)">true</xsl:when>
					<xsl:when test="ends-with($chkName,'ID')">true</xsl:when>
					<xsl:when test="ends-with($chkName,'INDICATOR')">true</xsl:when>
					<xsl:when test="ends-with($chkName,'NUMBER') and($maxLength &lt; 36)">true</xsl:when>
					<xsl:when test="ends-with($chkName,'NUMBERTEXT') and($maxLength &lt; 36)">true</xsl:when>
					<xsl:when test="ends-with($chkName,'NAME')"/>
					<xsl:when test="ends-with($chkName,'TEXT')"/>
					<xsl:when test="ends-with($chkName,'COMMENT')"/>
					<xsl:when test="ends-with($chkName,'DESCRIPTION')"/>
					<xsl:when test="$maxLength &lt; 21">true</xsl:when>
					<xsl:otherwise/>
				</xsl:choose> 
				</xsl:variable>
				<xsl:if test="$assignMask = 'true' or($useAll = 'true')"><as:constraint action="{concat('setLength(',$thisPath,',',$mask[1],')')}"/></xsl:if>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="setTypeSample">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:choose>
			<xsl:when test="$type=''"/>
			<xsl:when test="$type='date'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='datetime'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='time'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='integer'">%1234%</xsl:when>
			<xsl:when test="$type='decimal'">%654.10%</xsl:when>
			<xsl:when test="$type='float'">%7654.0001%</xsl:when>
			<xsl:when test="$type='anyURI'">%http://www.camtoolkit.com%</xsl:when>
			<xsl:otherwise>%<xsl:value-of select="$type"/>%</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="nsResolver">
		<xsl:param name="itemName"/>
		<xsl:param name="ns"/>
		<xsl:choose>
			<xsl:when test="count(tokenize($ns,'http:')) &gt; 1">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:value-of select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$ns=''">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:variable name="matchNS" select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
						<xsl:choose>
							<xsl:when test="string-length($matchNS) &lt; 1"><xsl:value-of select="concat('http://jcam.org.uk/ns/',$thisNS)"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$matchNS"/></xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="$ns"/></xsl:otherwise>
		</xsl:choose>	
	</xsl:template>
	
	<xsl:template match="*" mode="structureAnno">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		<!-- Choose to lookup item structure in dictionary, or simply skip  -->
		<xsl:variable name="parentChildPath" select="concat('/',$parent,'/',$thisItem)"/>
		<xsl:choose>
			<xsl:when test="contains($namespacesList,$thisNS)">
				<!-- Lookup in dictionary -->
				<xsl:variable name="importStructure">
					<xsl:choose>
						<!-- ignore items that are marked as already expanded -->
						<xsl:when test="$expansionItems/expander/camed:annotation[ends-with(@item,$contextMatch)]"><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:when><!-- ignore items already expanded -->
						<xsl:when test="$expansionItems/expander/camed:expand[ends-with(@item,$contextMatch)]"><!-- expand item with Type annotation -->
							<xsl:variable name="expandThis" select="concat('/',$expansionItems/expander/camed:expand[ends-with(@item,$contextMatch)]/@from)"/>
							<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@item = $expandThis]"/>
						</xsl:when>
						<xsl:when test="$dictionary//components/ITEMSlookup/Property[@item=$thisItem]">
							<!-- Find matching items -->
							<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@context = $parentChildPath]"/><!-- Get the parent node property -->
							<xsl:choose>				
								<!-- anno : this finds first item in dictionary -->
								<xsl:when test="count($dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]) &gt; 0">
									<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[starts-with(@context,$contextMatch)]"/>
								</xsl:when>
								<xsl:otherwise><!-- its a child item -->
									<xsl:variable name="childPathMatch" select="concat($contextMatch,'/')"/>
									<xsl:variable name="tempDefinition">
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[@item=$thisItem][1]"/>
										<xsl:copy-of select="$dictionary//components/ITEMSlookup/Property[contains(@context,$childPathMatch)]"/>
									</xsl:variable>
									
									<xsl:element name="Property">
										<xsl:for-each select="$tempDefinition//@*">
											<xsl:choose>
												<xsl:when test="name()='context'">
													<xsl:attribute name="context" select="concat('/',$thisItem,substring-after(.,$thisItem))"/>
												</xsl:when>
												<xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
											</xsl:choose>	
										</xsl:for-each>
									</xsl:element>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise><xsl:element name="no_match"><xsl:value-of select="$thisItem"/></xsl:element></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				
				<xsl:choose>
					<xsl:when test="$importStructure/no_match"><xsl:apply-templates mode="structureAnno"/></xsl:when><!-- Skip to next structure item -->
					<xsl:otherwise><!-- Output structure anno details from dictionary definition -->
						<xsl:variable name="importRest">
							<xsl:for-each select="$importStructure/Property">
								<xsl:choose>
									<xsl:when test="position()=1"/>
									<xsl:when test="@context=$contextMatch"><xsl:copy-of select="."/></xsl:when>
									<xsl:when test="@context=$parentChildPath"><xsl:copy-of select="."/></xsl:when>
									<xsl:when test="contains(@context,concat($contextMatch,'/'))"><xsl:copy-of select="."/></xsl:when>
									<xsl:otherwise/>
								</xsl:choose>
							</xsl:for-each>
						</xsl:variable>
						<xsl:variable name="uniqueAnnoList" select="distinct-values($importRest/Property/@context)"/>
						<xsl:variable name="importRestUnique">
							<xsl:for-each select="$uniqueAnnoList">
								<xsl:variable name="thisContextItem" select="."/>
								<xsl:copy-of select="$importRest/Property[@context=$thisContextItem][1]"/>
							</xsl:for-each>
						</xsl:variable>
						<xsl:apply-templates select="." mode="importAnno">
							<xsl:with-param name="this" select="$importStructure/Property[1]"/>
							<xsl:with-param name="parent" select="$parent"/>
							<xsl:with-param name="rest" select="$importRestUnique"/>
						</xsl:apply-templates>
						<xsl:apply-templates mode="structureAnno"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>				
				<xsl:apply-templates mode="structureAnno"/>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="*" mode="importAnno">
		<xsl:param name="this"/>
		<xsl:param name="parent"/>
		<xsl:param name="rest"/>
		<xsl:variable name="thisItem" select="$this/./@item"/>
		<xsl:variable name="thisItemType" select="$this/./@CCTS"/>
		<xsl:variable name="thisContext" select="$this/./@context"/>		
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="more" select="count($rest/Property)"/>
		<xsl:if test="$debug &gt; 0"><xsl:message>anno: [<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$parent"/>][<xsl:value-of select="$this/./@CCTS"/>][<xsl:value-of select="$thisContext"/>]</xsl:message></xsl:if>
		<xsl:variable name="parentContext" select="concat($this/./@context,'/')"/>
		<xsl:variable name="shortContext">
			<xsl:call-template name="shortContext">
				<xsl:with-param name="thisContext" select="$thisContext"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:choose>
			<!-- xsl:when test="$this/./@CCTS ='ASBIE'"/ --><!-- ignore attributes -->
			<xsl:when test="$more &gt; 0">
				<!-- Now insert the annotation from the definition details -->
				
				<xsl:if test="string-length($this/./@definition) &gt; 0">
					<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
						<xsl:attribute name="item" select="$shortContext"/>
						<camed:documentation type="Definition"><xsl:value-of select="$this/./@definition"/></camed:documentation>
						<camed:documentation type="Expanded">From <xsl:value-of select="$this/./@item"/> dictionary definition.</camed:documentation>
					</xsl:element>
				</xsl:if>				
				
				<xsl:for-each select="$rest/Property">
					<!-- xsl:message>[<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$parentContext"/>][<xsl:value-of select="@context"/>]</xsl:message -->
					<!-- Now see if there are children of this element -->
					<xsl:variable name="chkContext" select="concat(@context,'/')"/>
					<xsl:variable name="itemContext" select="substring-after(@context,$parentContext)"/>
					<xsl:variable name="next-namespace-uri" select="namespace-uri()"/>
					<xsl:variable name="children" select="count($rest/Property[starts-with(@context,$chkContext)])"/>
					<xsl:choose>
						<xsl:when test="$children &gt; 0">
							<!-- Process parent and children -->
							<xsl:variable name="importRest">
								<xsl:for-each select="$rest/Property[contains(@context,$chkContext)]">
									<xsl:copy-of select="."/>
								</xsl:for-each>
							</xsl:variable>
							<xsl:variable name="uniqueAnnoList" select="distinct-values($importRest/Property/@context)"/>
							<xsl:variable name="importRestUnique">
								<xsl:for-each select="$uniqueAnnoList">
									<xsl:variable name="thisContextItem" select="."/>
									<xsl:copy-of select="$importRest/Property[@context=$thisContextItem][1]"/>
								</xsl:for-each>
							</xsl:variable>
							<!-- xsl:message>parent: <xsl:value-of select="$thisItemType"/> - <xsl:value-of select="$parent"/></xsl:message -->
							<xsl:variable name="parentElement">
								<xsl:choose>
									<xsl:when test="$parent='as:Structure'"/>
									<xsl:when test="$thisItemType='ASBIE'">
										<xsl:choose>
											<xsl:when test="count(tokenize($thisItem,'/')) &lt; 3"><xsl:value-of select="$parent"/></xsl:when>
											<xsl:when test="ends-with($thisItemType,$thisItem)"><xsl:value-of select="$parent"/></xsl:when>
											<xsl:otherwise><xsl:value-of select="$thisItem"/></xsl:otherwise>
										</xsl:choose>
									</xsl:when>
									<xsl:otherwise><xsl:value-of select="$thisItem"/></xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
							<xsl:apply-templates select="." mode="importAnno">
								<xsl:with-param name="this" select="."/>
								<xsl:with-param name="parent" select="$parentElement"/>
								<xsl:with-param name="rest" select="$importRestUnique"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="contains($itemContext,'/')"><!-- Ignore child of parent item --></xsl:when>
						<xsl:otherwise><!-- Process this singleton element -->
							<xsl:if test="string-length(./@definition) &gt; 0">
								<xsl:variable name="shortContext">
									<xsl:call-template name="shortContext">
										<xsl:with-param name="thisContext" select="concat('/',./@context)"/>
										<xsl:with-param name="parent" select="$parent"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
									<xsl:attribute name="item" select="$shortContext"/>
									<camed:documentation type="Definition"><xsl:value-of select="./@definition"/></camed:documentation>
									<camed:documentation type="Expanded">From <xsl:value-of select="./@item"/> dictionary definition.</camed:documentation>
								</xsl:element>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				
			</xsl:when>
			<xsl:otherwise>
				<!-- Process this singleton element -->
					<xsl:variable name="shortContext">
						<xsl:call-template name="shortContext">
							<xsl:with-param name="thisContext" select="concat('/',$this/./@context)"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
						<xsl:attribute name="item" select="$shortContext"/>
						<xsl:if test="string-length($this/./@definition) &gt; 0"><camed:documentation type="Definition"><xsl:value-of select="$this/./@definition"/></camed:documentation></xsl:if>
						<camed:documentation type="Expanded">From <xsl:value-of select="$this/./@item"/> dictionary definition.</camed:documentation>
					</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="lookupAttrs">
		<xsl:param name="mode"/>
		<xsl:param name="item"/>
		<xsl:param name="contextPath"/>
		<!-- This checks for attributes of the main element in the dictionary and inserts them if found -->
		<xsl:choose>
			<xsl:when test="$mode='structure'">
				<xsl:variable name="findAttr" select="concat($contextPath,'@')"></xsl:variable>
				<xsl:variable name="findAttr1" select="concat($contextPath,'/@')"></xsl:variable>
				<xsl:variable name="theseAttrs" select="$dictionary//components/ITEMSlookup/Property[@CCTS='ASBIE'][contains(@context,$findAttr) or(contains(@context,$findAttr1))]"/>
				<xsl:if test="$debug &gt; 4"><xsl:message>attr find: <xsl:value-of select="$findAttr"/> [<xsl:value-of select="count($theseAttrs/Property)"/>]</xsl:message></xsl:if>
				<xsl:for-each select="$theseAttrs">
				    	<xsl:variable name="sampleValue"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="@type"/><xsl:with-param name="mask" select="@mask"/></xsl:call-template></xsl:variable>
				    	<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="@item"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
					<xsl:attribute name="{@item}" select="$sampleValue" namespace="{$actualNS}"/>
				</xsl:for-each>		
				<xsl:if test="count($theseAttrs) = 0">
					<xsl:variable name="thisPathItems" select="count(tokenize($contextPath,'/'))"/>
					<xsl:variable name="getFrontPath">
						<xsl:for-each select="tokenize($contextPath,'/')">
						<xsl:choose>
							<xsl:when test=".=''"/><!-- ignore blank -->
							<xsl:when test="position() = $thisPathItems - 1"><xsl:value-of select="concat('/',.)"/></xsl:when>
							<xsl:when test="position() = $thisPathItems"><xsl:value-of select="concat('/',.)"/></xsl:when>
							<xsl:otherwise/><!-- ignore long path items -->
						</xsl:choose>
						</xsl:for-each>
						<xsl:value-of select="'@'"/>
					</xsl:variable>
					<xsl:variable name="getFrontPath1" select="replace($getFrontPath,'@','/@')"/>
					
					<xsl:choose>
						<xsl:when test="count(tokenize($getFrontPath,'/')) &gt; 2">
							<xsl:variable name="moreAttrs" select="$dictionary//components/ITEMSlookup/Property[contains(@context,$getFrontPath) or(contains(@context,$getFrontPath1))]"/>
							<xsl:if test="$debug &gt; 4"><xsl:message>attr front: <xsl:value-of select="$getFrontPath"/> [<xsl:value-of select="count($moreAttrs)"/>]</xsl:message></xsl:if>
							<xsl:for-each select="$moreAttrs">
								<xsl:variable name="sampleValue"><xsl:call-template name="setTypeSample"><xsl:with-param name="type" select="@type"/><xsl:with-param name="mask" select="@mask"/></xsl:call-template></xsl:variable>
								<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="@item"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
								<xsl:attribute name="{@item}" select="$sampleValue" namespace="{$actualNS}"/>
							</xsl:for-each>	
						</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="shortContext"><!-- This routine takes the input XPath of the current node and shortens it to the two component form required for quirky Eclipse editor annotations -->
		<xsl:param name="thisContext"/>
		<xsl:param name="parent"/>
		<xsl:variable name="patchContext"><!-- Correct content of incoming XPath locator -->
			<xsl:choose>
				<xsl:when test="contains($thisContext,'@')"><xsl:value-of select="if(contains($thisContext,'/@')) then $thisContext else replace($thisContext,'@','/@')"/></xsl:when>
				<xsl:when test="contains($thisContext,'as:Structure/')"><xsl:value-of select="replace($thisContext,'as:Structure/','')"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$thisContext"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="contextPathNodes" select="tokenize($patchContext,'/')"/>
		<xsl:variable name="contextPathCount" select="count($contextPathNodes)"/>
		<xsl:variable name="shortContext">
			<xsl:choose>
				<xsl:when test="contains($thisContext,'@')"><xsl:value-of select="concat('//',$contextPathNodes[$contextPathCount - 1],'/',$contextPathNodes[$contextPathCount])"/></xsl:when>
				<xsl:when test="$contextPathCount &gt; 3"><xsl:value-of select="concat('//',$contextPathNodes[$contextPathCount - 1],'/',$contextPathNodes[$contextPathCount])"/></xsl:when>
				<xsl:when test="$parent='as:Structure'"><xsl:value-of select="replace(concat('/',$thisContext),'///','//')"/></xsl:when>
				<xsl:when test="not(contains($thisContext,concat('/',$parent,'/'))) and(not(ends-with($thisContext,concat('/',$parent))))"><xsl:value-of select="replace(concat('//',replace(concat($parent,$thisContext),'//','/')),'///','//')"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="replace(concat('//',$thisContext),'///','//')"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="if(contains($shortContext,'@') and not(contains($shortContext,'/@'))) then replace($shortContext,'@','/@') else $shortContext"/>
	</xsl:template>
	
	<xsl:template name="recursionDetect">
		<xsl:param name="parentItem"/>
		<xsl:param name="children"/>
		<!-- This inspects the children of the children to see which ones reference back to parentItem or parentItem type and flag accordingly -->
		<xsl:variable name="chkItem" select="concat(';',if(ends-with($parentItem,'Type')) then substring-before($parentItem,'Type') else $parentItem,'&#xA;')"/>
		<xsl:variable name="chkItemType" select="concat(';',if(ends-with($parentItem,'Type')) then $parentItem else concat($parentItem,'Type'),'&#xA;')"/>
		
		<xsl:for-each select="tokenize($children,'&#xA;')">
			<xsl:variable name="thisChild" select="."/>
			<xsl:if test="position() &gt; 1"> <!-- ignore context path -->
				<xsl:variable name="chkChildren" select="$dictionary//components/ITEMSlookup/Property[@item=$thisChild][1]/@children"/>
				<xsl:for-each select="tokenize($chkChildren[1],'&#xA;')">
					<xsl:variable name="chkChild" select="."/>
					<xsl:variable name="chkChildName" select="concat(';',.,'&#xA;')"/>
					<xsl:if test="position() &gt; 1"> <!-- ignore context path -->
						<xsl:if test="$chkChildName = $chkItem or($chkChildName = $chkItemType)">
							<xsl:value-of select="concat($thisChild,'|',$chkChild,'&#xA;')"/>
						</xsl:if>
						<xsl:variable name="theseChildren" select="$dictionary//components/ITEMSlookup/Property[@item=$chkChild][1]/@children"/>
						<xsl:for-each select="tokenize($theseChildren[1],'&#xA;')">
							<xsl:choose>
								<xsl:when test="position()=1"/><!-- ignore Parent XPath -->
								<xsl:when test=".=''"/>
								<xsl:when test=".=$chkItem or(.=$chkItemType)">
									<xsl:value-of select="concat($thisChild,' child|',$chkChild,'&#xA;')"/>
								</xsl:when>
								<xsl:otherwise/>
							</xsl:choose>
						</xsl:for-each>
						<!-- we may need to check children of children -->
					</xsl:if>
				</xsl:for-each>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:function name="camed:merge-item-lists">
		<xsl:param name="itemA"/>
		<xsl:param name="itemB"/>
		<xsl:param name="listA"/>
		<xsl:param name="listB"/>
		<xsl:param name="fullA"/>
		
		<xsl:choose>
			<xsl:when test="$itemA = '|' and ($itemB = '|')"/><!-- merge finished -->
			<xsl:when test="$itemA = $itemB"><!-- matched, so output and loop with next -->
				<xsl:variable name="nextA" select="substring-after($listA,$itemA)"/>
				<xsl:variable name="nextB" select="substring-after($listB,$itemB)"/>
				<xsl:value-of select="concat($itemA,camed:merge-item-lists(concat(substring-before($nextA,'|'),'|'),concat(substring-before($nextB,'|'),'|'),$nextA,$nextB,$fullA))"/>
			</xsl:when>
			<xsl:when test="$itemB = '|'"><xsl:value-of select="$listA"/></xsl:when><!-- more A items than B, output remaining A items -->
			<xsl:when test="$itemA = '|'"><xsl:value-of select="$listB"/></xsl:when><!-- more B items than A, output remaining B items -->
			<xsl:otherwise>
				<xsl:variable name="nextB" select="substring-after($listB,$itemB)"/>
				<xsl:choose>
					<xsl:when test="contains($fullA,concat('|',$itemB)) or (starts-with($fullA,$itemB))"><!-- skip B if its in A and loop with next B -->
						<xsl:value-of select="camed:merge-item-lists($itemA,concat(substring-before($nextB,'|'),'|'),$listA,$nextB,$fullA)"/>
					</xsl:when>
					<xsl:otherwise><!-- Output B and loop with next B -->
						<xsl:value-of select="concat($itemB,camed:merge-item-lists($itemA,concat(substring-before($nextB,'|'),'|'),$listA,$nextB,$fullA))"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:function>

</xsl:stylesheet>