<?xml version='1.0' ?>
<!DOCTYPE stylesheet [
<!ENTITY copy  "&#169;" >
<!ENTITY nbsp "&#160;"> 
<!ENTITY gte "&gt;=">
<!ENTITY lte "&lt;="> 
]>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:look="http://jcam.org.uk/LookupLists" xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:as="http://www.oasis-open.org/committees/cam">
	<!-- CAM kit utility program -->
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: David RR Webber, August, 2010 -->
	<!--
		Purpose  : Parse XSD schema into CAM lookup code list format.
		Creates CAM lookup files to be used by CAMV validation engine.
		Original : 08/30/2010
		Initials : drrw
		CAM ver    : 1.1 
		Description : Part of a set of tools to ease alignment of XSD based validations and documentation
		and CAM templates. 
		Updated  - Inits - Change
		08/30/10   drrw   Initial release
		12/28/10   drrw   Make more flexible in selecting enumeration lists from XSD source
		12/28/10   drrw   Improve handling of UTF8 and linefeed chars
		12/28/10   drrw   Handling UBL CodeName element in documentation
		12/28/10   drrw   Change extension to cll
		12/29/10   drrw   Change extension to cllr
	-->
	
	<xsl:param name="listsName"></xsl:param>
	<xsl:param name="outputPath"></xsl:param>
	<xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
	<xsl:output name="lookupContent" method="xml" indent="yes" omit-xml-declaration="yes" />
	<xsl:variable name="version">1.05</xsl:variable>
	
	<xsl:template match="/">
		<xsl:message> XSD Schema enumerations to CAM lookup utility</xsl:message>
		<xsl:message> Version: <xsl:value-of select="$version"/> - <xsl:value-of select="format-dateTime(current-dateTime() ,'[h01]:[m01]:[s01].[f01]')"/></xsl:message>
		<xsl:message> Writing <xsl:value-of select="count(xsd:schema/xsd:simpleType)"/> codelists:</xsl:message>
		<xsl:for-each select="xsd:schema/xsd:simpleType">
		       <xsl:if test="xsd:restriction/xsd:enumeration">
			<xsl:variable name="lookupFile"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"></xsl:value-of><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookup-<xsl:value-of select="@name"/>.cll</xsl:variable>
		       	<xsl:message> - <xsl:value-of select="$lookupFile"/> (<xsl:value-of select="count(xsd:restriction/xsd:enumeration)"></xsl:value-of>)</xsl:message>
			<xsl:result-document exclude-result-prefixes="xsd" format="lookupContent" indent="yes" href="{$lookupFile}">
				<look:LookupList xmlns:look="http://jcam.org.uk/LookupLists">
					<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
					<xsl:for-each select="xsd:restriction/xsd:enumeration">
						
						<xsl:variable name="UBLname" select="xsd:annotation/xsd:documentation"/>
						<!-- xsl:message><xsl:value-of select="$UBLname/*"/> count <xsl:value-of select="count($UBLname/child::*)"/></xsl:message -->
						<look:item>
							<xsl:attribute name="name">
								<xsl:choose>
									<xsl:when test="count($UBLname/child::*) &gt; 0"><xsl:value-of select="$UBLname/*"/></xsl:when>
									<xsl:when test="xsd:annotation/xsd:documentation"><xsl:value-of select="xsd:annotation/xsd:documentation"/></xsl:when>
									<xsl:otherwise><xsl:value-of select="@value"/>-name</xsl:otherwise>
								</xsl:choose>
							</xsl:attribute>
							<xsl:value-of select="@value"/>
						</look:item>
					</xsl:for-each>
				</look:LookupList>
			</xsl:result-document>
		        </xsl:if>
		</xsl:for-each>
		<!-- Now write out the includes extension section for these lookups -->
		<xsl:variable name="controlFile">
			<xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"></xsl:value-of><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>
			<xsl:choose>
				<xsl:when test="string-length($listsName) &gt; 0"><xsl:value-of select="$listsName"/><xsl:if test="not(ends-with($listsName,'.lists') or(ends-with($listsName,'.xml')))">.cllr</xsl:if></xsl:when>
				<xsl:otherwise>all-lookup-lists-include.cllr</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:message>Writing includes file to: </xsl:message>
		<xsl:message> - <xsl:value-of select="$controlFile"/></xsl:message>
		<xsl:if test="xsd:schema/xsd:simpleType/xsd:restriction/xsd:enumeration">		
		    <xsl:result-document exclude-result-prefixes="xsd" format="lookupContent" indent="yes" href="{$controlFile}">
		       <as:Extension
		    		xmlns:as="http://www.oasis-open.org/committees/cam" name="uk.org.jcam.camed.extensions.LookupLists">
		       <xsl:for-each select="xsd:schema/xsd:simpleType">
			<xsl:if test="xsd:restriction/xsd:enumeration">
				<as:include>lookup-<xsl:value-of select="@name"/>.cll</as:include>
			</xsl:if>
		       </xsl:for-each>
		       </as:Extension>
		    </xsl:result-document>
		</xsl:if>
		<xsl:message>Processing complete.</xsl:message>
	</xsl:template>
	<xsl:template match="text()"/>
</xsl:stylesheet>